@@ -9,14 +9,18 @@
 
 #include <config.h>
 
+#ifdef __linux
 #include <sys/prctl.h>
 #include <sys/syscall.h>
+#endif
 #include <unistd.h>
 
 #include <atomic>
 #include <cassert>
+#include <cstring>
 #include <ctime>
 #include <iomanip>
+#include <iostream>
 #include <sstream>
 #include <string>
 
@@ -33,28 +37,26 @@
 #include "Log.hpp"
 #include "Util.hpp"
 
-static char LogPrefix[256] = { '\0' };
-
 namespace Log
 {
     using namespace Poco;
 
     /// Helper to avoid destruction ordering issues.
-    struct StaticNames
+    struct StaticNameHelper
     {
         std::atomic<bool> inited;
         std::string name;
         std::string id;
-        StaticNames() :
+        StaticNameHelper() :
             inited(true)
         {
         }
-        ~StaticNames()
+        ~StaticNameHelper()
         {
             inited = false;
         }
     };
-    static StaticNames Source;
+    static StaticNameHelper Source;
 
     // We need a signal safe means of writing messages
     //   $ man 7 signal
@@ -62,7 +64,7 @@ namespace Log
     {
         while (true)
         {
-            const int length = strlen(message);
+            const int length = std::strlen(message);
             const int written = write (STDERR_FILENO, message, length);
             if (written < 0)
             {
@@ -80,7 +82,7 @@ namespace Log
 
     // We need a signal safe means of writing messages
     //   $ man 7 signal
-    void signalLogNumber(size_t num)
+    void signalLogNumber(std::size_t num)
     {
         int i;
         char buf[22];
@@ -93,26 +95,16 @@ namespace Log
         signalLog(buf + i + 1);
     }
 
-    char* prefix(char* buffer, const char* level, bool sigSafe)
+    char* prefix(char* buffer, const std::size_t len, const char* level)
     {
-        long osTid;
-        char procName[32];
-        const char *threadName = procName;
-        if (sigSafe)
-        {
-            osTid = syscall(SYS_gettid);
-
-            if (prctl(PR_GET_NAME, reinterpret_cast<unsigned long>(procName), 0, 0, 0) != 0)
-                strncpy(procName, "<noid>", sizeof(procName) - 1);
-        }
-        else
-        {
-            osTid = Util::getThreadId();
-            threadName = Util::getThreadName();
-        }
-
+        const char *threadName = Util::getThreadName();
+#ifdef __linux
+        const long osTid = Util::getThreadId();
+#elif defined IOS
+        const auto osTid = pthread_mach_thread_np(pthread_self());
+#endif
         Poco::DateTime time;
-        snprintf(buffer, 1023, "%s-%.05lu %.4u-%.2u-%.2u %.2u:%.2u:%.2u.%.6u [ %s ] %s  ",
+        snprintf(buffer, len, "%s-%.05lu %.4u-%.2u-%.2u %.2u:%.2u:%.2u.%.6u [ %s ] %s  ",
                     (Source.inited ? Source.id.c_str() : "<shutdown>"),
                     osTid,
                     time.year(), time.month(), time.day(),
@@ -125,7 +117,7 @@ namespace Log
     void signalLogPrefix()
     {
         char buffer[1024];
-        prefix(buffer, "SIG", true);
+        prefix(buffer, sizeof(buffer) - 1, "SIG");
         signalLog(buffer);
     }
 
@@ -133,15 +125,16 @@ namespace Log
                     const std::string& logLevel,
                     const bool withColor,
                     const bool logToFile,
-                    std::map<std::string, std::string> config)
+                    const std::map<std::string, std::string>& config)
     {
         Source.name = name;
         std::ostringstream oss;
-        oss << Source.name << '-'
+        oss << Source.name;
+#ifndef MOBILEAPP // Just one process in a mobile app, the pid is uninteresting.
+        oss << '-'
             << std::setw(5) << std::setfill('0') << Poco::Process::id();
+#endif
         Source.id = oss.str();
-        assert (sizeof (LogPrefix) > strlen(oss.str().c_str()) + 1);
-        strncpy(LogPrefix, oss.str().c_str(), sizeof(LogPrefix));
 
         // Configure the logger.
         AutoPtr<Channel> channel;
@@ -190,6 +183,17 @@ namespace Log
     {
         return Poco::Logger::get(Source.inited ? Source.name : std::string());
     }
+
+    void shutdown()
+    {
+        logger().shutdown();
+
+        // Flush
+        std::flush(std::cout);
+        fflush(stdout);
+        std::flush(std::cerr);
+        fflush(stderr);
+    }
 }
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
