@@ -12,6 +12,9 @@
  */
 
 #include <config.h>
+
+#include "Seccomp.hpp"
+
 #include <dlfcn.h>
 #include <ftw.h>
 #include <linux/audit.h>
@@ -24,13 +27,13 @@
 #include <sys/capability.h>
 #include <sys/prctl.h>
 #include <sys/resource.h>
+#include <sys/syscall.h>
 #include <sys/time.h>
 #include <unistd.h>
 #include <utime.h>
 
 #include <common/Log.hpp>
 #include <common/SigUtil.hpp>
-#include "Seccomp.hpp"
 
 #if DISABLE_SECCOMP == 0
 #ifndef SYS_SECCOMP
@@ -71,6 +74,7 @@ static void handleSysSignal(int /* signal */,
 
     SigUtil::dumpBacktrace();
 
+    Log::shutdown();
     _exit(1);
 }
 
@@ -101,9 +105,7 @@ bool lockdown(Type type)
         // Load sycall number
         BPF_STMT(BPF_LD+BPF_W+BPF_ABS,  offsetof(struct seccomp_data, nr)),
 
-        // ------------------------------------------------------------
-        // ---   First white-list the syscalls we frequently use.   ---
-        // ------------------------------------------------------------
+        // First white-list the syscalls we frequently use.
         ACCEPT_SYSCALL(recvfrom),
         ACCEPT_SYSCALL(write),
         ACCEPT_SYSCALL(futex),
@@ -115,9 +117,7 @@ bool lockdown(Type type)
         ACCEPT_SYSCALL(close),
         ACCEPT_SYSCALL(nanosleep),
 
-        // ------------------------------------------------------------
-        // --- Now block everything that we don't like the look of. ---
-        // ------------------------------------------------------------
+        // Now block everything that we don't like the look of.
 
         // FIXME: should we bother blocking calls that have early
         // permission checks we don't meet ?
@@ -210,7 +210,7 @@ bool lockdown(Type type)
 
     sigemptyset(&action.sa_mask);
     action.sa_flags = SA_SIGINFO;
-    action.sa_handler = reinterpret_cast<__sighandler_t>(handleSysSignal);
+    action.sa_sigaction = handleSysSignal;
 
     sigaction(SIGSYS, &action, nullptr);
 
