@@ -13,8 +13,12 @@
 #include <atomic>
 #include <mutex>
 
+#ifndef MOBILEAPP
 /// Flag to commence clean shutdown
 extern std::atomic<bool> ShutdownRequestFlag;
+#else
+static constexpr bool ShutdownRequestFlag(false);
+#endif
 
 /// Flag to stop pump loops.
 extern std::atomic<bool> TerminationFlag;
@@ -22,6 +26,8 @@ extern std::atomic<bool> TerminationFlag;
 /// Flag to dump internal state
 extern std::atomic<bool> DumpGlobalState;
 
+#ifndef MOBILEAPP
+
 /// Mutex to trap signal handler, if any,
 /// and prevent _Exit while collecting backtrace.
 extern std::mutex SigHandlerTrap;
@@ -61,6 +67,8 @@ namespace SigUtil
 
 } // end namespace SigUtil
 
+#endif // !MOBILEAPP
+
 #endif
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
