@@ -85,7 +85,7 @@ public:
             {
                 if (TerminationFlag)
                 {
-                    LOG_INF("Poll interrupted in " << getName() << " and Termination flag set.");
+                    LOG_INF("Poll interrupted in " << getName() << " and TerminationFlag is set.");
                 }
 
                 // Break.
@@ -221,6 +221,11 @@ static void cleanupChildren()
             LOG_INF("Child " << exitedChildPid << " has exited, will remove its jail [" << it->second << "].");
             jails.emplace_back(it->second);
             childJails.erase(it);
+            if (childJails.empty() && !TerminationFlag)
+            {
+                // We ran out of kits and we aren't terminating.
+                LOG_WRN("No live Kits exist, and we are not terminating yet.");
+            }
         }
         else
         {
@@ -264,7 +269,7 @@ static int createLibreOfficeKit(const std::string& childRoot,
             const size_t delaySecs = std::stoul(std::getenv("SLEEPKITFORDEBUGGER"));
             if (delaySecs > 0)
             {
-                std::cerr << "Sleeping " << delaySecs
+                std::cerr << "Kit: Sleeping " << delaySecs
                           << " seconds to give you time to attach debugger to process "
                           << Process::id() << std::endl;
                 Thread::sleep(delaySecs * 1000);
@@ -353,7 +358,7 @@ int main(int argc, char** argv)
         const size_t delaySecs = std::stoul(std::getenv("SLEEPFORDEBUGGER"));
         if (delaySecs > 0)
         {
-            std::cerr << "Sleeping " << delaySecs
+            std::cerr << "Forkit: Sleeping " << delaySecs
                       << " seconds to give you time to attach debugger to process "
                       << Process::id() << std::endl;
             Thread::sleep(delaySecs * 1000);
@@ -382,7 +387,7 @@ int main(int argc, char** argv)
     LogLevel = logLevel ? logLevel : "trace";
     if (LogLevel != "trace")
     {
-        LOG_INF("Setting log-level to [trace] and delaying setting to requested [" << LogLevel << "].");
+        LOG_INF("Setting log-level to [trace] and delaying setting to configured [" << LogLevel << "] until after Forkit initialization.");
     }
 
     std::string childRoot;
@@ -447,7 +452,7 @@ int main(int argc, char** argv)
             std::vector<std::string> tokens = LOOLProtocol::tokenize(rlimits, ';');
             for (const std::string& cmdLimit : tokens)
             {
-                const std::pair<std::string, std::string> pair = LOOLProtocol::split(cmdLimit, ':');
+                const std::pair<std::string, std::string> pair = Util::split(cmdLimit, ':');
                 std::vector<std::string> tokensLimit = { "setconfig", pair.first, pair.second };
                 if (!Rlimit::handleSetrlimitCommand(tokensLimit))
                 {
@@ -525,7 +530,11 @@ int main(int argc, char** argv)
 
     // Initialize LoKit
     if (!globalPreinit(loTemplate))
+    {
+        LOG_FTL("Failed to preinit lokit.");
+        Log::shutdown();
         std::_Exit(Application::EXIT_SOFTWARE);
+    }
 
     if (Util::getProcessThreadCount() != 1)
         LOG_ERR("Error: forkit has more than a single thread after pre-init");
@@ -539,6 +548,7 @@ int main(int argc, char** argv)
     if (forKitPid < 0)
     {
         LOG_FTL("Failed to create a kit process.");
+        Log::shutdown();
         std::_Exit(Application::EXIT_SOFTWARE);
     }
 
@@ -546,7 +556,7 @@ int main(int argc, char** argv)
     ::unsetenv("LOOL_TRACE_STARTUP");
     if (LogLevel != "trace")
     {
-        LOG_INF("Setting log-level to [" << LogLevel << "].");
+        LOG_INF("Forkit initialization complete: setting log-level to [" << LogLevel << "] as configured.");
         Log::logger().setLevel(LogLevel);
     }
 
@@ -575,6 +585,7 @@ int main(int argc, char** argv)
 #endif
 
     LOG_INF("ForKit process finished.");
+    Log::shutdown();
     std::_Exit(returnValue);
 }
 #endif
