@@ -1,9 +1,37 @@
-/* -*- js-indent-level: 8 -*- */
+/* -*- js-indent-level: 8; fill-column: 100 -*- */
 /*
  * L.Socket contains methods for the communication with the server
  */
 
 /* global _ vex $ errorMessages Uint8Array brandProductName brandProductFAQURL */
+
+window.fakeWebSocketCounter = 0;
+function FakeWebSocket() {
+	this.binaryType = 'arraybuffer';
+	this.bufferedAmount = 0;
+	this.extensions = '';
+	this.protocol = '';
+	this.readyState = 1;
+	this.id = window.fakeWebSocketCounter++;
+	this.sendCounter = 0;
+	this.onclose = function() {
+	};
+	this.onerror = function() {
+	};
+	this.onmessage = function() {
+	};
+	this.onopen = function() {
+	};
+}
+
+FakeWebSocket.prototype.close = function() {
+}
+
+FakeWebSocket.prototype.send = function(data) {
+	this.sendCounter++;
+	window.webkit.messageHandlers.lool.postMessage(data, '*');
+}
+
 L.Socket = L.Class.extend({
 	ProtocolVersionNumber: '0.1',
 	ReconnectCount: 0,
@@ -27,27 +55,32 @@ L.Socket = L.Class.extend({
 		if (map.options.permission) {
 			map.options.docParams['permission'] = map.options.permission;
 		}
-		var wopiSrc = '';
-		if (map.options.wopiSrc != '') {
-			wopiSrc = '?WOPISrc=' + map.options.wopiSrc + '&compat=/ws';
+		if (this.socket) {
+			this.close();
 		}
-
-		var websocketURI = map.options.server + '/lool/' + encodeURIComponent(map.options.doc + '?' + $.param(map.options.docParams)) + '/ws' + wopiSrc;
-		try {
-			if (this.socket) {
-				this.close();
+		if (window.ThisIsAMobileApp) {
+			this.socket = new FakeWebSocket();
+			window.TheFakeWebSocket = this.socket;
+		} else {
+			var wopiSrc = '';
+			if (map.options.wopiSrc != '') {
+				wopiSrc = '?WOPISrc=' + map.options.wopiSrc + '&compat=/ws';
 			}
-			this.socket = new WebSocket(websocketURI);
-		} catch (e) {
-			// On IE 11 there is a limitation on the number of WebSockets open to a single domain (6 by default and can go to 128).
-			// Detect this and hint the user.
-			var msgHint = '';
-			var isIE11 = !!window.MSInputMethodContext && !!document.documentMode; // https://stackoverflow.com/questions/21825157/internet-explorer-11-detection
-			if (isIE11)
-				msgHint = _('IE11 has reached its maximum number of connections. Please see this document to increase this limit if needed: https://docs.microsoft.com/en-us/previous-versions/windows/internet-explorer/ie-developer/general-info/ee330736(v=vs.85)#websocket-maximum-server-connections');
 
-			this._map.fire('error', {msg: _('Oops, there is a problem connecting to LibreOffice Online : ').replace('LibreOffice Online', (typeof brandProductName !== 'undefined' ? brandProductName : 'LibreOffice Online')) + e + msgHint, cmd: 'socket', kind: 'failed', id: 3});
-			return;
+			try {
+				var websocketURI = map.options.server + map.options.serviceRoot + '/lool/' + encodeURIComponent(map.options.doc + '?' + $.param(map.options.docParams)) + '/ws' + wopiSrc;
+				this.socket = new WebSocket(websocketURI);
+			} catch (e) {
+				// On IE 11 there is a limitation on the number of WebSockets open to a single domain (6 by default and can go to 128).
+				// Detect this and hint the user.
+				var msgHint = '';
+				var isIE11 = !!window.MSInputMethodContext && !!document.documentMode; // https://stackoverflow.com/questions/21825157/internet-explorer-11-detection
+				if (isIE11)
+					msgHint = _('IE11 has reached its maximum number of connections. Please see this document to increase this limit if needed: https://docs.microsoft.com/en-us/previous-versions/windows/internet-explorer/ie-developer/general-info/ee330736(v=vs.85)#websocket-maximum-server-connections');
+
+				this._map.fire('error', {msg: _('Oops, there is a problem connecting to LibreOffice Online : ').replace('LibreOffice Online', (typeof brandProductName !== 'undefined' ? brandProductName : 'LibreOffice Online')) + e + msgHint, cmd: 'socket', kind: 'failed', id: 3});
+				return;
+			}
 		}
 
 		this.socket.onerror = L.bind(this._onSocketError, this);
@@ -55,7 +88,17 @@ L.Socket = L.Class.extend({
 		this.socket.onopen = L.bind(this._onSocketOpen, this);
 		this.socket.onmessage = L.bind(this._onMessage, this);
 		this.socket.binaryType = 'arraybuffer';
-
+		if (window.ThisIsAMobileApp) {
+			// This corresponds to the initial GET request when creating a WebSocket
+			// connection and tells the app's code that it is OK to start invoking
+			// TheFakeWebSocket's onmessage handler. Should we also include the
+			// map.options.doc, as in the websocketURI above? On the other hand, the app
+			// code that handles this special message knows the document to be edited
+			// anyway, and can send it on as necessary to the Online code.
+			window.webkit.messageHandlers.lool.postMessage('HULLO', '*');
+			// A FakeWebSocket is immediately open.
+			this.socket.onopen();
+		}
 		if (map.options.docParams.access_token && parseInt(map.options.docParams.access_token_ttl)) {
 			var tokenExpiryWarning = 900 * 1000; // Warn when 15 minutes remain
 			clearTimeout(this._accessTokenExpireTimeout);
@@ -518,6 +561,7 @@ L.Socket = L.Class.extend({
 
 			if (this._map._docLayer) {
 				this._map._docLayer.removeAllViews();
+				this._map._docLayer._resetClientVisArea();
 			}
 			this.close();
 
@@ -741,7 +785,8 @@ L.Socket = L.Class.extend({
 					if (command.type === 'presentation' &&
 					    this._map.options.defaultZoom === this._map.options.zoom) {
 						// If we have a presentation document and the zoom level has not been set
-						// in the options, resize the document so that it fits the viewing area
+						// in the options, resize the document so that it fits the viewing area.
+						// FIXME: Should this 256 be window.tileSize? Unclear to me.
 						var verticalTiles = this._map.getSize().y / 256;
 						tileWidthTwips = Math.round(command.height / verticalTiles);
 						tileHeightTwips = Math.round(command.height / verticalTiles);
@@ -761,7 +806,8 @@ L.Socket = L.Class.extend({
 			else if (this._reconnecting) {
 				// we are reconnecting ...
 				this._reconnecting = false;
-				this._map._docLayer._onMessage('invalidatetiles: EMPTY', null);
+				this._map._docLayer._resetClientVisArea();
+				this._map._docLayer._requestNewTiles();
 				this._map.fire('statusindicator', {statusType: 'reconnected'});
 				this._map.setPermission(this._map.options.permission);
 			}
@@ -804,6 +850,7 @@ L.Socket = L.Class.extend({
 
 		if (this._map._docLayer) {
 			this._map._docLayer.removeAllViews();
+			this._map._docLayer._resetClientVisArea();
 		}
 
 		if (isActive && this._reconnecting) {
