@@ -8,7 +8,7 @@ L.GridLayer = L.Layer.extend({
 	options: {
 		pane: 'tilePane',
 
-		tileSize: 256,
+		tileSize: window.tileSize,
 		opacity: 1,
 
 		updateWhenIdle: L.Browser.mobile,
@@ -24,6 +24,8 @@ L.GridLayer = L.Layer.extend({
 
 	initialize: function (options) {
 		L.setOptions(this, options);
+
+		this._resetClientVisArea()
 	},
 
 	onAdd: function () {
@@ -331,6 +333,7 @@ L.GridLayer = L.Layer.extend({
 		    tileZoomChanged = this._tileZoom !== tileZoom;
 
 		if (!noUpdate && (hard || tileZoomChanged)) {
+			this._resetClientVisArea();
 
 			if (this._abortLoading) {
 				this._abortLoading();
@@ -356,6 +359,14 @@ L.GridLayer = L.Layer.extend({
 		this._setZoomTransforms(center, zoom);
 	},
 
+	// These variables indicates the clientvisiblearea sent to the server and stored by the server
+	// We need to reset them when we are reconnecting to the server or reloading a document
+	// because the server needs new data even if the client is unmodified.
+	_resetClientVisArea: function ()  {
+		this._clientZoom = '';
+		this._clientVisibleArea = '';
+	},
+
 	_updateTileTwips: function () {
 		// smaller zoom = zoom in
 		var factor = Math.pow(1.2, (this._map.options.zoom - this._tileZoom));
@@ -529,15 +540,23 @@ L.GridLayer = L.Layer.extend({
 
 				key = this._tileCoordsToKey(coords);
 				var tile = this._tiles[key];
-				if (tile) {
+				var invalid = tile && tile._invalidCount && tile._invalidCount > 0;
+				if (tile && tile.loaded && !invalid) {
 					tile.current = true;
 					newView = false;
+				} else if (invalid) {
+					tile._invalidCount = 1;
+					queue.push(coords);
 				} else {
 					queue.push(coords);
 				}
 			}
 		}
 
+		this._sendClientVisibleArea(true);
+
+		this._sendClientZoom(true);
+
 		if (queue.length !== 0) {
 			if (newView) {
 				// we know that a new set of tiles that cover the whole view has been requested
@@ -684,6 +703,55 @@ L.GridLayer = L.Layer.extend({
 		}
 	},
 
+	_requestNewTiles: function () {
+		this._onMessage('invalidatetiles: EMPTY', null);
+		this._update();
+	},
+
+	toggleTileDebugMode: function() {
+		this.toggleTileDebugModeImpl();
+		this._requestNewTiles();
+	},
+
+	_sendClientVisibleArea: function (forceUpdate) {
+		var visibleTopLeft = this._latLngToTwips(this._map.getBounds().getNorthWest());
+		var visibleBottomRight = this._latLngToTwips(this._map.getBounds().getSouthEast());
+		var visibleArea = new L.Bounds(visibleTopLeft, visibleBottomRight);
+		var size = new L.Point(visibleArea.getSize().x, visibleArea.getSize().y);
+		var newClientVisibleArea = 'clientvisiblearea x=' + Math.round(visibleTopLeft.x) + ' y=' + Math.round(visibleTopLeft.y) +
+			' width=' + Math.round(size.x) + ' height=' + Math.round(size.y);
+
+		if (this._clientVisibleArea !== newClientVisibleArea || forceUpdate) {
+			// Visible area is dirty, update it on the server
+			this._map._socket.sendMessage(newClientVisibleArea);
+			if (!this._map._fatal && this._map._active && this._map._socket.connected())
+				this._clientVisibleArea = newClientVisibleArea
+			if (this._debug) {
+				this._debugInfo.clearLayers();
+				for (var key in this._tiles) {
+					this._tiles[key]._debugPopup = null;
+					this._tiles[key]._debugTile = null;
+				}
+			}
+		}
+	},
+
+	_sendClientZoom: function (forceUpdate) {
+
+		var newClientZoom = 'tilepixelwidth=' + this._tileWidthPx + ' ' +
+			'tilepixelheight=' + this._tileHeightPx + ' ' +
+			'tiletwipwidth=' + this._tileWidthTwips + ' ' +
+			'tiletwipheight=' + this._tileHeightTwips;
+
+		if (this._clientZoom !== newClientZoom || forceUpdate) {
+			// the zoom level has changed
+			this._map._socket.sendMessage('clientzoom ' + newClientZoom);
+
+			if (!this._map._fatal && this._map._active && this._map._socket.connected())
+				this._clientZoom = newClientZoom;
+		}
+	},
+
 	_cancelTiles: function() {
 		this._map._socket.sendMessage('canceltiles');
 		for (var key in this._tiles) {
@@ -824,39 +892,42 @@ L.GridLayer = L.Layer.extend({
 			key = this._tileCoordsToKey(coords);
 
 			if (coords.part === this._selectedPart) {
-				var tile = this.createTile(this._wrapCoords(coords), L.bind(this._tileReady, this, coords));
+				if (!this._tiles[key]) {
+					var tile = this.createTile(this._wrapCoords(coords), L.bind(this._tileReady, this, coords));
 
-				this._initTile(tile);
+					this._initTile(tile);
 
-				// if createTile is defined with a second argument ("done" callback),
-				// we know that tile is async and will be ready later; otherwise
-				if (this.createTile.length < 2) {
-					// mark tile as ready, but delay one frame for opacity animation to happen
-					setTimeout(L.bind(this._tileReady, this, coords, null, tile), 0);
+					// if createTile is defined with a second argument ("done" callback),
+					// we know that tile is async and will be ready later; otherwise
+					if (this.createTile.length < 2) {
+						// mark tile as ready, but delay one frame for opacity animation to happen
+						setTimeout(L.bind(this._tileReady, this, coords, null, tile), 0);
+					}
+
+					// we prefer top/left over translate3d so that we don't create a HW-accelerated layer from each tile
+					// which is slow, and it also fixes gaps between tiles in Safari
+					L.DomUtil.setPosition(tile, tilePos, true);
+
+					// save tile in cache
+					this._tiles[key] = {
+						el: tile,
+						coords: coords,
+						current: true
+					};
+
+					fragment.appendChild(tile);
+
+					this.fire('tileloadstart', {
+						tile: tile,
+						coords: coords
+					});
+
+					if (this._tileCache[key]) {
+						tile.src = this._tileCache[key];
+					}
 				}
-
-				// we prefer top/left over translate3d so that we don't create a HW-accelerated layer from each tile
-				// which is slow, and it also fixes gaps between tiles in Safari
-				L.DomUtil.setPosition(tile, tilePos, true);
-
-				// save tile in cache
-				this._tiles[key] = {
-					el: tile,
-					coords: coords,
-					current: true
-				};
-
-				fragment.appendChild(tile);
-
-				this.fire('tileloadstart', {
-					tile: tile,
-					coords: coords
-				});
 			}
 
-			if (this._tileCache[key]) {
-				tile.src = this._tileCache[key];
-			}
 		}
 
 		// sort the tiles by the rows
@@ -932,53 +1003,33 @@ L.GridLayer = L.Layer.extend({
 		var twips, msg;
 		for (var r = 0; r < rectangles.length; ++r) {
 			rectQueue = rectangles[r];
-
-			if (rectQueue.length === 1) {
-				// only one tile here
-				coords = rectQueue[0];
-				key = this._tileCoordsToKey(coords);
-
+			var tilePositionsX = '';
+			var tilePositionsY = '';
+			for (i = 0; i < rectQueue.length; i++) {
+				coords = rectQueue[i];
 				twips = this._coordsToTwips(coords);
-				msg = 'tile ' +
-					'part=' + coords.part + ' ' +
-					'width=' + this._tileWidthPx + ' ' +
-					'height=' + this._tileHeightPx + ' ' +
-					'tileposx=' + twips.x + ' '	+
-					'tileposy=' + twips.y + ' ' +
-					'tilewidth=' + this._tileWidthTwips + ' ' +
-					'tileheight=' + this._tileHeightTwips;
-				this._map._socket.sendMessage(msg, key);
-			}
-			else {
-				// more tiles, use tilecombine
-				var tilePositionsX = '';
-				var tilePositionsY = '';
-				for (i = 0; i < rectQueue.length; i++) {
-					coords = rectQueue[i];
-					twips = this._coordsToTwips(coords);
 
-					if (tilePositionsX !== '') {
-						tilePositionsX += ',';
-					}
-					tilePositionsX += twips.x;
-
-					if (tilePositionsY !== '') {
-						tilePositionsY += ',';
-					}
-					tilePositionsY += twips.y;
+				if (tilePositionsX !== '') {
+					tilePositionsX += ',';
 				}
+				tilePositionsX += twips.x;
 
-				twips = this._coordsToTwips(coords);
-				msg = 'tilecombine ' +
-					'part=' + coords.part + ' ' +
-					'width=' + this._tileWidthPx + ' ' +
-					'height=' + this._tileHeightPx + ' ' +
-					'tileposx=' + tilePositionsX + ' '	+
-					'tileposy=' + tilePositionsY + ' ' +
-					'tilewidth=' + this._tileWidthTwips + ' ' +
-					'tileheight=' + this._tileHeightTwips;
-				this._map._socket.sendMessage(msg, '');
+				if (tilePositionsY !== '') {
+					tilePositionsY += ',';
+				}
+				tilePositionsY += twips.y;
 			}
+
+			twips = this._coordsToTwips(coords);
+			msg = 'tilecombine ' +
+				'part=' + coords.part + ' ' +
+				'width=' + this._tileWidthPx + ' ' +
+				'height=' + this._tileHeightPx + ' ' +
+				'tileposx=' + tilePositionsX + ' '	+
+				'tileposy=' + tilePositionsY + ' ' +
+				'tilewidth=' + this._tileWidthTwips + ' ' +
+				'tileheight=' + this._tileHeightTwips;
+			this._map._socket.sendMessage(msg, '');
 		}
 	},
 
