@@ -65,7 +65,7 @@ L.TileLayer = L.GridLayer.extend({
 		this._tileHeightPx = options.tileSize;
 
 		// detecting retina displays, adjusting tileWidthPx, tileHeightPx and zoom levels
-		if (options.docType !== 'spreadsheet' && options.detectRetina && L.Browser.retina && options.maxZoom > 0) {
+		if (options.detectRetina && L.Browser.retina && options.maxZoom > 0) {
 			this._tileWidthPx *= 2;
 			this._tileHeightPx *= 2;
 			options.zoomOffset++;
@@ -87,10 +87,10 @@ L.TileLayer = L.GridLayer.extend({
 		this._documentInfo = '';
 		// Position and size of the visible cursor.
 		this._visibleCursor = new L.LatLngBounds(new L.LatLng(0, 0), new L.LatLng(0, 0));
-		// Cursor overlay is visible or hidden (for blinking).
-		this._isCursorOverlayVisible = false;
-		// Cursor overlay visibility flag to store last state during zooming
-		this._oldCursorOverlayVisibility = false;
+		// Do we have focus - ie. should we render a cursor
+		this._isFocused = true;
+		// Are we zooming currently ? - if so, no cursor.
+		this._isZooming = false;
 		// Cursor is visible or hidden (e.g. for graphic selection).
 		this._isCursorVisible = true;
 		// Original rectangle graphic selection in twips
@@ -149,18 +149,11 @@ L.TileLayer = L.GridLayer.extend({
 		this._msgQueue = [];
 		this._toolbarCommandValues = {};
 		this._previewInvalidations = [];
-		this._clientZoom = 'tilepixelwidth=' + this._tileWidthPx + ' ' +
-			'tilepixelheight=' + this._tileHeightPx + ' ' +
-			'tiletwipwidth=' + this.options.tileWidthTwips + ' ' +
-			'tiletwipheight=' + this.options.tileHeightTwips;
 
 		this._followThis = -1;
 		this._editorId = -1;
 		this._followUser = false;
 		this._followEditor = false;
-
-		// Mark visible area as dirty by default.
-		this._invalidateClientVisibleArea();
 	},
 
 	onAdd: function (map) {
@@ -262,9 +255,7 @@ L.TileLayer = L.GridLayer.extend({
 		if (this._docType === 'spreadsheet') {
 			map.on('zoomend', this._onCellCursorShift, this);
 		}
-		map.on('zoomend', this._updateClientZoom, this);
 		map.on('zoomend', L.bind(this.eachView, this, this._viewCursors, this._onUpdateViewCursor, this, false));
-		map.on('resize zoomend', this._invalidateClientVisibleArea, this);
 		map.on('dragstart', this._onDragStart, this);
 		map.on('requestloksession', this._onRequestLOKSession, this);
 		map.on('error', this._mapOnError, this);
@@ -291,7 +282,6 @@ L.TileLayer = L.GridLayer.extend({
 		for (var key in this._selectionHandles) {
 			this._selectionHandles[key].on('drag dragend', this._onSelectionHandleDrag, this);
 		}
-		this._map.focus();
 
 		map.setPermission(this.options.permission);
 
@@ -492,10 +482,43 @@ L.TileLayer = L.GridLayer.extend({
 		else if (textMsg.startsWith('validitylistbutton:')) {
 			this._onValidityListButtonMsg(textMsg);
 		}
+		else if (textMsg.startsWith('signaturestatus:')) {
+			var signstatus = textMsg.substring('signaturestatus:'.length + 1);
+
+			if (signstatus == '1') {
+				this._map.fire('infobar',
+					{
+						msg: 'Document signed and validated.', action: null, actionLabel: null
+					});
+			}
+			else if (signstatus == '2') {
+				this._map.fire('infobar',
+					{
+						msg: 'Document signed but signatue is broken.', action: null, actionLabel: null
+					});
+			}
+			else if (signstatus == '3') {
+				this._map.fire('infobar',
+					{
+						msg: 'Document signed but the document is already modified.', action: null, actionLabel: null
+					});
+			}
+			else if (signstatus == '4') {
+				this._map.fire('infobar',
+					{
+						msg: 'Document signed but can not be validated.', action: null, actionLabel: null
+					});
+			}
+			else if (signstatus == '5') {
+				this._map.fire('infobar',
+					{
+						msg: 'Document signed but not all files are signed.', action: null, actionLabel: null
+					});
+			}
+		}
 	},
 
-	toggleTileDebugMode: function() {
-		this._invalidateClientVisibleArea();
+	toggleTileDebugModeImpl: function() {
 		this._debug = !this._debug;
 		if (!this._debug) {
 			this._map.removeLayer(this._debugInfo);
@@ -509,7 +532,6 @@ L.TileLayer = L.GridLayer.extend({
 			}
 			this._debugInit();
 		}
-		this._onMessage('invalidatetiles: EMPTY', null);
 	},
 
 	_onCommandValuesMsg: function (textMsg) {
@@ -555,7 +577,6 @@ L.TileLayer = L.GridLayer.extend({
 	_onCursorVisibleMsg: function(textMsg) {
 		var command = textMsg.match('cursorvisible: true');
 		this._isCursorVisible = command ? true : false;
-		this._isCursorOverlayVisible = true;
 		this._onUpdateCursor();
 	},
 
@@ -568,14 +589,19 @@ L.TileLayer = L.GridLayer.extend({
 		if (this._map.options.wopiSrc != '') {
 			wopiSrc = '?WOPISrc=' + this._map.options.wopiSrc;
 		}
-		var url = this._map.options.webserver + '/' + this._map.options.urlPrefix + '/' +
+		var url = this._map.options.webserver + this._map.options.serviceRoot + '/' + this._map.options.urlPrefix + '/' +
 		    encodeURIComponent(this._map.options.doc) + '/' + command.jail + '/' + command.dir + '/' + command.name + wopiSrc;
 
 		this._map.hideBusy();
 		if (command.id === 'print') {
 			if (L.Browser.gecko || L.Browser.edge || this._map.options.print === false) {
 				// the print dialog doesn't work well on firefox
-				this._map.fire('print', {url: url});
+				// due to a pdf.js issue - https://github.com/mozilla/pdf.js/issues/5397
+				// open the pdf file in a new tab so that that user can print it directly in the browser's
+				// pdf viewer
+				var param = wopiSrc !== '' ? '&' : '?';
+				param += 'attachment=0';
+				window.open(url + param, '_blank');
 			}
 			else {
 				this._map.fire('filedownloadready', {url: url});
@@ -764,8 +790,6 @@ L.TileLayer = L.GridLayer.extend({
 		this._visibleCursor = new L.LatLngBounds(
 						this._twipsToLatLng(topLeftTwips, this._map.getZoom()),
 						this._twipsToLatLng(bottomRightTwips, this._map.getZoom()));
-		this._visibleCursorOnLostFocus = this._visibleCursor;
-		this._isCursorOverlayVisible = true;
 		if ((docLayer._followEditor || docLayer._followUser) && this._map.lastActionByUser) {
 			this._map.fire('setFollowOff');
 		}
@@ -1334,8 +1358,8 @@ L.TileLayer = L.GridLayer.extend({
 		else if (tile && typeof(img) == 'object') {
 			// 'Uint8Array' delta
 			var canvas = document.createElement('canvas');
-			canvas.width = 256;
-			canvas.height = 256;
+			canvas.width = window.tileSize;
+			canvas.height = window.tileSize;
 			var ctx = canvas.getContext('2d');
 
 			var oldImg = new Image();
@@ -1408,17 +1432,11 @@ L.TileLayer = L.GridLayer.extend({
 			}
 
 			ctx.putImageData(imgData, 0, 0);
-
-			tile.oldWireId = tile.wireId;
-			tile.wireId = command.wireId;
 			tile.el.src = canvas.toDataURL('image/png');
 
 			console.log('set new image');
 		}
 		else if (tile) {
-			if (command.wireId != undefined) {
-				tile.oldWireId = command.wireId;
-			}
 			if (this._tiles[key]._invalidCount > 0) {
 				this._tiles[key]._invalidCount -= 1;
 			}
@@ -1431,6 +1449,10 @@ L.TileLayer = L.GridLayer.extend({
 			tile.el.src = img;
 		}
 		L.Log.log(textMsg, L.INCOMING, key);
+
+		// Send acknowledgment, that the tile message arrived
+		var tileID = command.part + ':' + command.x + ':' + command.y + ':' + command.tileWidth + ':' + command.tileHeight;
+		this._map._socket.sendMessage('tileprocessed tile=' + tileID);
 	},
 
 	_tileOnLoad: function (done, tile) {
@@ -1456,8 +1478,7 @@ L.TileLayer = L.GridLayer.extend({
 	},
 
 	_clearSelections: function () {
-		// hide the cursor
-		this._isCursorOverlayVisible = false;
+		// hide the cursor if not editable
 		this._onUpdateCursor();
 		// hide the text selection
 		this._selections.clearLayers();
@@ -1470,16 +1491,29 @@ L.TileLayer = L.GridLayer.extend({
 		this._onUpdateCellCursor();
 	},
 
-	_postMouseEvent: function(type, x, y, count, buttons, modifier) {
-		if (this._clientZoom) {
-			// the zoom level has changed
-			this._map._socket.sendMessage('clientzoom ' + this._clientZoom);
-			this._clientZoom = null;
+	containsSelection: function (latlng) {
+		var ret = false;
+		var selections = this._selections.getLayers();
+		for (var sel in selections) {
+			if (selections[sel].getBounds().contains(latlng)) {
+				ret = true;
+				break;
+			}
 		}
+		return ret;
+	},
+
+	_postMouseEvent: function(type, x, y, count, buttons, modifier) {
+
+		this._sendClientZoom();
+
+		this._sendClientVisibleArea();
+
 		this._map._socket.sendMessage('mouse type=' + type +
 				' x=' + x + ' y=' + y + ' count=' + count +
 				' buttons=' + buttons + ' modifier=' + modifier);
 
+
 		if (type === 'buttondown') {
 			this._clearSearchResults();
 		}
@@ -1500,21 +1534,11 @@ L.TileLayer = L.GridLayer.extend({
 				this._cellCursorOnPgDn = new L.LatLngBounds(this._prevCellCursor.getSouthWest(), this._prevCellCursor.getNorthEast());
 			}
 		}
-		if (this._clientZoom) {
-			// the zoom level has changed
-			this._map._socket.sendMessage('clientzoom ' + this._clientZoom);
-			this._clientZoom = null;
-		}
-		if (this._clientVisibleArea) {
-			// Visible area is dirty, update it on the server.
-			var visibleArea = this._map._container.getBoundingClientRect();
-			var pos = this._pixelsToTwips(new L.Point(visibleArea.left, visibleArea.top));
-			var size = this._pixelsToTwips(new L.Point(visibleArea.width, visibleArea.height));
-			var payload = 'clientvisiblearea x=' + Math.round(pos.x) + ' y=' + Math.round(pos.y) +
-				' width=' + Math.round(size.x) + ' height=' + Math.round(size.y);
-			this._map._socket.sendMessage(payload);
-			this._clientVisibleArea = false;
-		}
+
+		this._sendClientZoom();
+
+		this._sendClientVisibleArea();
+
 		this._map._socket.sendMessage('key type=' + type +
 				' char=' + charcode + ' key=' + keycode);
 	},
@@ -1543,15 +1567,16 @@ L.TileLayer = L.GridLayer.extend({
 	},
 
 	_onZoomStart: function () {
-		this._oldCursorOverlayVisibility = this._isCursorOverlayVisible;
-		this._isCursorOverlayVisible = false;
+		this._isZooming = true;
 		this._onUpdateCursor();
+		this.updateAllViewCursors();
 	},
 
 
 	_onZoomEnd: function () {
-		this._isCursorOverlayVisible = this._oldCursorOverlayVisibility;
+		this._isZooming = false;
 		this._onUpdateCursor();
+		this.updateAllViewCursors();
 	},
 
 	_updateCursorPos: function () {
@@ -1566,8 +1591,7 @@ L.TileLayer = L.GridLayer.extend({
 			this._cursorMarker.setLatLng(cursorPos, pixBounds.getSize().multiplyBy(this._map.getZoomScale(this._map.getZoom())));
 		}
 
-		this._map.addLayer(this._cursorMarker);
-		return this._visibleCursor.getNorthWest();
+		this._map._clipboardContainer.showCursor();
 	},
 
 	// Update cursor layer (blinking cursor).
@@ -1584,7 +1608,7 @@ L.TileLayer = L.GridLayer.extend({
 			if (!(this._selectionHandles.start && this._selectionHandles.start.isDragged) &&
 			    !(this._selectionHandles.end && this._selectionHandles.end.isDragged) &&
 			    !(docLayer._followEditor || docLayer._followUser)) {
-				this._map.fire('scrollto', {x: center.x, y: center.y});
+				this._map.fire('scrollto', {x: center.x, y: center.y, calledFromInvalidateCursorMsg: scroll !== undefined});
 			}
 		}
 
@@ -1593,7 +1617,7 @@ L.TileLayer = L.GridLayer.extend({
 		this.eachView(this._viewCursors, function (item) {
 			var viewCursorMarker = item.marker;
 			if (viewCursorMarker) {
-				viewCursorMarker.setOpacity(this._map.hasLayer(this._cursorMarker) && this._cursorMarker.getLatLng().equals(viewCursorMarker.getLatLng()) ? 0 : 1);
+				viewCursorMarker.setOpacity(this.isCursorVisible() && this._cursorMarker.getLatLng().equals(viewCursorMarker.getLatLng()) ? 0 : 1);
 			}
 		}, this, true);
 	},
@@ -1602,15 +1626,15 @@ L.TileLayer = L.GridLayer.extend({
 	// the state of the document (if the falgs are set)
 	_updateCursorAndOverlay: function (/*update*/) {
 		if (this._map._permission === 'edit'
-		&& this._map._clipboardContainer._textArea === document.activeElement
-		&& this._isCursorVisible
-		&& this._isCursorOverlayVisible
+		&& this._isCursorVisible        // only when LOK has told us it is ok
+		&& this._isFocused              // not when document is not focused
+		&& !this._isZooming             // not when zooming
+		&& !this.isGraphicVisible()     // not when sizing / positioning graphics
 		&& !this._isEmptyRectangle(this._visibleCursor)) {
 			this._updateCursorPos();
 		}
 		else if (this._cursorMarker) {
 			this._map.removeLayer(this._cursorMarker);
-			this._isCursorOverlayVisible = false;
 		}
 	},
 
@@ -1628,8 +1652,11 @@ L.TileLayer = L.GridLayer.extend({
 		var viewCursorVisible = this._viewCursors[viewId].visible;
 		var viewPart = this._viewCursors[viewId].part;
 
-		if (!this._map.isViewReadOnly(viewId) && viewCursorVisible && !this._isEmptyRectangle(this._viewCursors[viewId].bounds) &&
-		   (this._docType === 'text' || this._selectedPart === viewPart)) {
+		if (!this._map.isViewReadOnly(viewId) &&
+		    viewCursorVisible &&
+		    !this._isZooming &&
+		    !this._isEmptyRectangle(this._viewCursors[viewId].bounds) &&
+		    (this._docType === 'text' || this._selectedPart === viewPart)) {
 			if (!viewCursorMarker) {
 				var viewCursorOptions = {
 					color: L.LOUtil.rgbToHex(this._map.getViewColor(viewId)),
@@ -1645,7 +1672,7 @@ L.TileLayer = L.GridLayer.extend({
 			else {
 				viewCursorMarker.setLatLng(viewCursorPos, pixBounds.getSize().multiplyBy(this._map.getZoomScale(this._map.getZoom())));
 			}
-			viewCursorMarker.setOpacity(this._map.hasLayer(this._cursorMarker) && this._cursorMarker.getLatLng().equals(viewCursorMarker.getLatLng()) ? 0 : 1);
+			viewCursorMarker.setOpacity(this.isCursorVisible() && this._cursorMarker.getLatLng().equals(viewCursorMarker.getLatLng()) ? 0 : 1);
 			this._viewLayerGroup.addLayer(viewCursorMarker);
 		}
 		else if (viewCursorMarker) {
@@ -1653,6 +1680,20 @@ L.TileLayer = L.GridLayer.extend({
 		}
 	},
 
+	updateAllViewCursors : function() {
+		for (var key in this._viewCursors) {
+			this._onUpdateViewCursor(key);
+		}
+	},
+
+	isCursorVisible: function() {
+		return this._map.hasLayer(this._cursorMarker);
+	},
+
+	isGraphicVisible: function() {
+		return this._graphicMarker && this._map.hasLayer(this._graphicMarker);
+	},
+
 	goToViewCursor: function(viewId) {
 		if (viewId === this._viewId) {
 			this._onUpdateCursor();
@@ -1819,7 +1860,6 @@ L.TileLayer = L.GridLayer.extend({
 			this._graphicMarker = L.rectangle(this._graphicSelection, {
 				pointerEvents: 'none',
 				fill: false});
-			this._visibleCursor = this._visibleCursorOnLostFocus = this._graphicMarker._bounds;
 			if (!this._graphicMarker) {
 				this._map.fire('error', {msg: 'Graphic marker initialization', cmd: 'marker', kind: 'failed', id: 1});
 				return;
@@ -1834,6 +1874,7 @@ L.TileLayer = L.GridLayer.extend({
 			this._map.removeLayer(this._graphicMarker);
 			this._graphicMarker.isDragged = false;
 		}
+		this._updateCursorAndOverlay();
 	},
 
 	_onUpdateCellCursor: function (horizontalDirection, verticalDirection, onPgUpDn) {
@@ -2126,12 +2167,22 @@ L.TileLayer = L.GridLayer.extend({
 		}
 
 		// now try various mime types
-		var mimeTypes = [
-			['text/rtf', 'text/rtf'],
-			['text/html', 'text/html'],
-			['text/plain', 'text/plain;charset=utf-8'],
-			['Text', 'text/plain;charset=utf-8']
-		];
+		var mimeTypes;
+		if (this._docType === 'spreadsheet') {
+			// FIXME apparently we cannot paste the text/html or text/rtf as
+			// produced by LibreOffice in Calc from some reason
+			mimeTypes = [
+				['text/plain', 'text/plain;charset=utf-8'],
+				['Text', 'text/plain;charset=utf-8']
+			];
+		} else {
+			mimeTypes = [
+				['text/rtf', 'text/rtf'],
+				['text/html', 'text/html'],
+				['text/plain', 'text/plain;charset=utf-8'],
+				['Text', 'text/plain;charset=utf-8']
+			];
+		}
 
 		for (var i = 0; i < mimeTypes.length; ++i) {
 			for (t = 0; t < types.length; ++t) {
@@ -2279,24 +2330,6 @@ L.TileLayer = L.GridLayer.extend({
 		this._previewInvalidations = [];
 	},
 
-	_updateClientZoom: function () {
-		this._clientZoom = 'tilepixelwidth=' + this._tileWidthPx + ' ' +
-			'tilepixelheight=' + this._tileHeightPx + ' ' +
-			'tiletwipwidth=' + this._tileWidthTwips + ' ' +
-			'tiletwipheight=' + this._tileHeightTwips;
-	},
-
-	_invalidateClientVisibleArea: function() {
-		if (this._debug) {
-			this._debugInfo.clearLayers();
-			for (var key in this._tiles) {
-				this._tiles[key]._debugPopup = null;
-				this._tiles[key]._debugTile = null;
-			}
-		}
-		this._clientVisibleArea = true;
-	},
-
 	_debugGetTimeArray: function() {
 		return {count: 0, ms: 0, best: Number.MAX_SAFE_INTEGER, worst: 0, date: 0};
 	},
@@ -2451,7 +2484,7 @@ L.TileLayer = L.GridLayer.extend({
 					rect.setStyle({fillOpacity: opac - 0.04});
 				}
 			}
-			this._debugTimeoutId = setTimeout(function () { this._map._docLayer._debugTimeout(); }, 50);
+			this._debugTimeoutId = setTimeout(L.bind(this._debugTimeout, this), 50);
 		}
 	},
 
@@ -2464,7 +2497,7 @@ L.TileLayer = L.GridLayer.extend({
 			this._postKeyboardEvent('input', this._debugLorem.charCodeAt(this._debugLoremPos % this._debugLorem.length), 0);
 		}
 		this._debugLoremPos++;
-		this._debugTypeTimeoutId = setTimeout(function () { this._map._docLayer._debugTypeTimeout(); }, 50);
+		this._debugTypeTimeoutId = setTimeout(L.bind(this._debugTypeTimeout, this), 50);
 	}
 
 });
