@@ -27,6 +27,7 @@
 #include <countloolkits.hpp>
 #include <helpers.hpp>
 #include <test.hpp>
+#include <sstream>
 
 using namespace helpers;
 
@@ -58,9 +59,9 @@ class TileCacheTests : public CPPUNIT_NS::TestFixture
 
     CPPUNIT_TEST(testSimple);
     CPPUNIT_TEST(testSimpleCombine);
-    CPPUNIT_TEST(testPerformance);
     CPPUNIT_TEST(testCancelTiles);
-    CPPUNIT_TEST(testCancelTilesMultiView);
+    // unstable
+    // CPPUNIT_TEST(testCancelTilesMultiView);
     CPPUNIT_TEST(testDisconnectMultiView);
     CPPUNIT_TEST(testUnresponsiveClient);
     CPPUNIT_TEST(testImpressTiles);
@@ -78,12 +79,20 @@ class TileCacheTests : public CPPUNIT_NS::TestFixture
     // temporarily disable
     //CPPUNIT_TEST(testTileInvalidatePartCalc);
     //CPPUNIT_TEST(testTileInvalidatePartImpress);
+    CPPUNIT_TEST(testTileRequestByInvalidation);
+    CPPUNIT_TEST(testTileRequestByZoom);
+    CPPUNIT_TEST(testTileWireIDHandling);
+    CPPUNIT_TEST(testTileProcessed);
+    CPPUNIT_TEST(testTileInvalidatedOutside);
+    CPPUNIT_TEST(testTileBeingRenderedHandling);
+    CPPUNIT_TEST(testWireIDFilteringOnWSDSide);
+    CPPUNIT_TEST(testLimitTileVersionsOnFly);
+
 
     CPPUNIT_TEST_SUITE_END();
 
     void testSimple();
     void testSimpleCombine();
-    void testPerformance();
     void testCancelTiles();
     void testCancelTilesMultiView();
     void testDisconnectMultiView();
@@ -101,6 +110,14 @@ class TileCacheTests : public CPPUNIT_NS::TestFixture
     void testTileInvalidateCalc();
     void testTileInvalidatePartCalc();
     void testTileInvalidatePartImpress();
+    void testTileRequestByInvalidation();
+    void testTileRequestByZoom();
+    void testTileWireIDHandling();
+    void testTileProcessed();
+    void testTileInvalidatedOutside();
+    void testTileBeingRenderedHandling();
+    void testWireIDFilteringOnWSDSide();
+    void testLimitTileVersionsOnFly();
 
     void checkTiles(std::shared_ptr<LOOLWebSocket>& socket,
                     const std::string& type,
@@ -166,7 +183,7 @@ void TileCacheTests::testSimple()
 
     // Create TileCache and pretend the file was modified as recently as
     // now, so it discards the cached data.
-    TileCache tc("doc.ods", Poco::Timestamp(), "/tmp/tile_cache_tests");
+    TileCache tc("doc.ods", Poco::Timestamp(), "/tmp/tile_cache_tests", true);
 
     int part = 0;
     int width = 256;
@@ -234,29 +251,6 @@ void TileCacheTests::testSimpleCombine()
     CPPUNIT_ASSERT_MESSAGE("did not receive a tile: message as expected", !tile2b.empty());
 }
 
-void TileCacheTests::testPerformance()
-{
-    std::shared_ptr<LOOLWebSocket> socket = loadDocAndGetSocket("hello.odt", _uri, "performance ");
-
-    Poco::Timestamp timestamp;
-    for (int x = 0; x < 5; ++x)
-    {
-        sendTextFrame(socket, "tilecombine part=0 width=256 height=256 tileposx=0,3840,7680,11520,0,3840,7680,11520 tileposy=0,0,0,0,3840,3840,3840,3840 tilewidth=3840 tileheight=3840");
-        for (int i = 0; i < 8; ++i)
-        {
-            std::vector<char> tile = getResponseMessage(socket, "tile:", "tile-performance ");
-            CPPUNIT_ASSERT_MESSAGE("did not receive a tile: message as expected", !tile.empty());
-        }
-    }
-
-    std::cerr << "Tile rendering roundtrip for 5 x 8 tiles combined: " << timestamp.elapsed() / 1000.
-              << " ms. Per-tilecombine: " << timestamp.elapsed() / (1000. * 5)
-              << " ms. Per-tile: " << timestamp.elapsed() / (1000. * 5 * 8) << "ms."
-              << std::endl;
-
-    socket->shutdown();
-}
-
 void TileCacheTests::testCancelTiles()
 {
     const char* testName = "cancelTiles ";
@@ -430,6 +424,8 @@ void TileCacheTests::testUnresponsiveClient()
             std::vector<char> tile = getResponseMessage(socket2, "tile:", "client2 ");
             CPPUNIT_ASSERT_MESSAGE("Did not receive tile #" + std::to_string(i+1) + " of 8: message as expected", !tile.empty());
         }
+        /// Send canceltiles message to clear tiles-on-fly list, otherwise wsd waits for tileprocessed messages
+        sendTextFrame(socket2, "canceltiles");
     }
 }
 
@@ -1145,6 +1141,407 @@ void TileCacheTests::requestTiles(std::shared_ptr<LOOLWebSocket>& socket, const
     }
 }
 
+void TileCacheTests::testTileRequestByInvalidation()
+{
+    const char* testname = "tileRequestByInvalidation ";
+
+    std::string documentPath, documentURL;
+    getDocumentPathAndURL("empty.odt", documentPath, documentURL, testname);
+    std::shared_ptr<LOOLWebSocket> socket = loadDocAndGetSocket(_uri, documentURL, testname);
+
+    // 1. use case: invalidation without having a valid visible area in wsd
+    // Type one character to trigger invalidation
+    sendChar(socket, 'x', skNone, testname);
+
+    // First wsd forwards the invalidation
+    assertResponseString(socket, "invalidatetiles:", testname);
+
+    // Since we did not set client visible area wsd won't send tile
+    std::vector<char> tile = getResponseMessage(socket, "tile:", testname);
+    CPPUNIT_ASSERT_MESSAGE("Not expected tile message arrived!", tile.empty());
+
+    // 2. use case: invalidation of one tile inside the client visible area
+    // Now set the client visible area
+    sendTextFrame(socket, "clientvisiblearea x=-4005 y=0 width=50490 height=72300");
+    sendTextFrame(socket, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3840 tiletwipheight=3840");
+
+    // Type one character to trigger invalidation
+    sendChar(socket, 'x', skNone, testname);
+
+    // First wsd forwards the invalidation
+    assertResponseString(socket, "invalidatetiles:", testname);
+
+    // Then sends the new tile which was invalidated inside the visible area
+    assertResponseString(socket, "tile:", testname);
+}
+
+void TileCacheTests::testTileRequestByZoom()
+{
+    // By zoom the client requests all the tile of the visible area
+    // Server should push all these tiles to the network, so tiles-on-fly should be bigger than this count
+
+    const char* testname = "testTileRequestByZoom ";
+
+    std::string documentPath, documentURL;
+    getDocumentPathAndURL("empty.odt", documentPath, documentURL, testname);
+    std::shared_ptr<LOOLWebSocket> socket = loadDocAndGetSocket(_uri, documentURL, testname);
+
+    // Set the client visible area
+    sendTextFrame(socket, "clientvisiblearea x=-2662 y=0 width=16000 height=9875");
+    sendTextFrame(socket, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3200 tiletwipheight=3200");
+
+    // Request all tile of the visible area (it happens by zoom)
+    sendTextFrame(socket, "tilecombine part=0 width=256 height=256 tileposx=0,3200,6400,9600,12800,0,3200,6400,9600,12800,0,3200,6400,9600,12800,0,3200,6400,9600,12800 tileposy=0,0,0,0,0,3200,3200,3200,3200,3200,6400,6400,6400,6400,6400,9600,9600,9600,9600,9600 tilewidth=3200 tileheight=3200");
+
+    // Check that we get all the tiles without we send back the tileprocessed message
+    for (int i = 0; i < 20; ++i)
+    {
+        std::vector<char> tile = getResponseMessage(socket, "tile:", testname);
+        CPPUNIT_ASSERT_MESSAGE("Did not get tile as expected!", !tile.empty());
+    }
+}
+
+void TileCacheTests::testTileWireIDHandling()
+{
+    const char* testname = "testTileWireIDHandling ";
+
+    std::string documentPath, documentURL;
+    getDocumentPathAndURL("empty.odt", documentPath, documentURL, testname);
+    std::shared_ptr<LOOLWebSocket> socket = loadDocAndGetSocket(_uri, documentURL, testname);
+
+    // Set the client visible area
+    sendTextFrame(socket, "clientvisiblearea x=-4005 y=0 width=50490 height=72300");
+    sendTextFrame(socket, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3840 tiletwipheight=3840");
+
+    // Type one character to trigger invalidation
+    sendChar(socket, 'x', skNone, testname);
+
+    // First wsd forwards the invalidation
+    assertResponseString(socket, "invalidatetiles:", testname);
+
+    // For the first input wsd will send all invalidated tiles
+    int arrivedTiles = 0;
+    bool gotTile = false;
+    do
+    {
+        std::vector<char> tile = getResponseMessage(socket, "tile:", testname);
+        gotTile = !tile.empty();
+        if(gotTile)
+            ++arrivedTiles;
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_MESSAGE("We expect two tiles at least!", arrivedTiles > 1);
+
+    // Type an other character
+    sendChar(socket, 'x', skNone, testname);
+    assertResponseString(socket, "invalidatetiles:", testname);
+
+    // For the second input wsd will send one tile, since some of them are indentical
+    arrivedTiles = 0;
+    gotTile = false;
+    do
+    {
+        std::vector<char> tile = getResponseMessage(socket, "tile:", testname);
+        gotTile = !tile.empty();
+        if(gotTile)
+            ++arrivedTiles;
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_EQUAL(1, arrivedTiles);
+}
+
+void TileCacheTests::testTileProcessed()
+{
+    // Test whether tileprocessed message removes the tiles from the internal tiles-on-fly list
+    const char* testname = "testTileProcessed ";
+
+    std::string documentPath, documentURL;
+    getDocumentPathAndURL("empty.odt", documentPath, documentURL, testname);
+    std::shared_ptr<LOOLWebSocket> socket = loadDocAndGetSocket(_uri, documentURL, testname);
+
+    // Set the client visible area
+    sendTextFrame(socket, "clientvisiblearea x=-2662 y=0 width=10000 height=9000");
+    sendTextFrame(socket, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3200 tiletwipheight=3200");
+
+    // Request a lots of tiles (more than wsd can send once)
+    sendTextFrame(socket, "tilecombine part=0 width=256 height=256 tileposx=0,3200,6400,9600,12800,0,3200,6400,9600,12800,0,3200,6400,9600,12800,0,3200,6400,9600,12800,0,3200,6400,9600,12800 tileposy=0,0,0,0,0,3200,3200,3200,3200,3200,6400,6400,6400,6400,6400,9600,9600,9600,9600,9600,12800,12800,12800,12800,12800 tilewidth=3200 tileheight=3200");
+
+    std::vector<std::string> tileIDs;
+    int arrivedTile = 0;
+    bool gotTile = false;
+    do
+    {
+        std::string tile = getResponseString(socket, "tile:", testname);
+        gotTile = !tile.empty();
+        if(gotTile)
+        {
+            ++arrivedTile;
+
+            // Store tileID, so we can send it back
+            Poco::StringTokenizer tokens(tile, " ", Poco::StringTokenizer::TOK_IGNORE_EMPTY | Poco::StringTokenizer::TOK_TRIM);
+            std::string tileID = tokens[1].substr(std::string("part=").size()) + ":" +
+                                 tokens[4].substr(std::string("tileposx=").size()) + ":" +
+                                 tokens[5].substr(std::string("tileposy=").size()) + ":" +
+                                 tokens[6].substr(std::string("tileWidth=").size()) + ":" +
+                                 tokens[7].substr(std::string("tileHeight=").size());
+            tileIDs.push_back(tileID);
+        }
+
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_MESSAGE("We expect two tiles at least!", arrivedTile > 1);
+    CPPUNIT_ASSERT_MESSAGE("We expect that wsd can't send all the tiles!", arrivedTile < 25);
+
+    for(std::string& tileID : tileIDs)
+    {
+        sendTextFrame(socket, "tileprocessed tile=" + tileID);
+    }
+
+    // Now we can get the remaining tiles
+    int arrivedTile2 = 0;
+    gotTile = false;
+    do
+    {
+        std::vector<char> tile = getResponseMessage(socket, "tile:", testname);
+        gotTile = !tile.empty();
+        if(gotTile)
+            ++arrivedTile2;
+
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_MESSAGE("We expect one tile at least!", arrivedTile2 > 1);
+}
+
+void TileCacheTests::testTileInvalidatedOutside()
+{
+    // Test whether wsd sends us the tiles which are hanging out the visible area
+    const char* testname = "testTileInvalidatedOutside ";
+
+    std::string documentPath, documentURL;
+    getDocumentPathAndURL("empty.odt", documentPath, documentURL, testname);
+    std::shared_ptr<LOOLWebSocket> socket = loadDocAndGetSocket(_uri, documentURL, testname);
+
+    // Type one character to trigger invalidation and get the invalidation rectangle
+    sendChar(socket, 'x', skNone, testname);
+
+    // First wsd forwards the invalidation
+    std::string sInvalidate = assertResponseString(socket, "invalidatetiles:", testname);
+    Poco::StringTokenizer tokens(sInvalidate, " ", Poco::StringTokenizer::TOK_IGNORE_EMPTY | Poco::StringTokenizer::TOK_TRIM);
+    int y = std::stoi(tokens[3].substr(std::string("y=").size()));
+    int height = std::stoi(tokens[5].substr(std::string("height=").size()));
+
+
+    // Set client visible area to make it not having intersection with the invalidate rectangle, but having shared tiles
+    std::ostringstream oss;
+    oss << "clientvisiblearea x=0 y=" << (y + height + 100) << " width=50490 height=72300";
+    sendTextFrame(socket, oss.str());
+    sendTextFrame(socket, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3840 tiletwipheight=3840");
+
+    // Type one character to trigger invalidation
+    sendChar(socket, 'x', skNone, testname);
+
+    // First wsd forwards the invalidation
+    assertResponseString(socket, "invalidatetiles:", testname);
+
+    // Since the invalidation rectangle is outside the visible area
+    // wsd does not send a new tile even if some of the invalidated tiles
+    // are partly visible.
+    std::vector<char> tile = getResponseMessage(socket, "tile:", testname);
+    CPPUNIT_ASSERT_MESSAGE("Not expected tile message arrived!", tile.empty());
+}
+
+void TileCacheTests::testTileBeingRenderedHandling()
+{
+    // The issue here was that we requested the tile of the same tile twice
+    // and so sometimes we got the same tile message twice from wsd.
+    const char* testname = "testTileBeingRenderedHandling ";
+
+    std::string documentPath, documentURL;
+    getDocumentPathAndURL("empty.odt", documentPath, documentURL, testname);
+    std::shared_ptr<LOOLWebSocket> socket = loadDocAndGetSocket(_uri, documentURL, testname);
+
+    // Set the client visible area
+    sendTextFrame(socket, "clientvisiblearea x=-2662 y=0 width=16000 height=9875");
+    sendTextFrame(socket, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3200 tiletwipheight=3200");
+
+    // Type one character to trigger invalidation
+    sendChar(socket, 'x', skNone, testname);
+
+    // First wsd forwards the invalidation
+    assertResponseString(socket, "invalidatetiles:", testname);
+
+    // For the first input wsd will send all invalidated tiles
+    int arrivedTiles = 0;
+    bool gotTile = false;
+    do
+    {
+        std::vector<char> tile = getResponseMessage(socket, "tile:", testname);
+        gotTile = !tile.empty();
+        if(gotTile)
+            ++arrivedTiles;
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_MESSAGE("We expect two tiles at least!", arrivedTiles > 1);
+
+    // For the later inputs wsd will send one tile, since other ones are indentical
+    for(int i = 0; i < 5; ++i)
+    {
+        // Type an other character
+        sendChar(socket, 'x', skNone, testname);
+        assertResponseString(socket, "invalidatetiles:", testname);
+
+        arrivedTiles = 0;
+        gotTile = false;
+        do
+        {
+            std::vector<char> tile = getResponseMessage(socket, "tile:", testname, 500);
+            gotTile = !tile.empty();
+            if(gotTile)
+                ++arrivedTiles;
+        } while(gotTile);
+
+        CPPUNIT_ASSERT_EQUAL(1, arrivedTiles);
+
+        sendTextFrame(socket, "tileprocessed tile=0:0:0:3200:3200");
+    }
+}
+
+void TileCacheTests::testWireIDFilteringOnWSDSide()
+{
+    const char* testname = "testWireIDFilteringOnWSDSide ";
+
+    std::string documentPath, documentURL;
+    getDocumentPathAndURL("empty.odt", documentPath, documentURL, testname);
+    std::shared_ptr<LOOLWebSocket> socket1 = loadDocAndGetSocket(_uri, documentURL, testname);
+    // Set the client visible area
+    sendTextFrame(socket1, "clientvisiblearea x=-4005 y=0 width=50490 height=72300");
+    sendTextFrame(socket1, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3840 tiletwipheight=3840");
+
+    std::shared_ptr<LOOLWebSocket> socket2 = loadDocAndGetSocket(_uri, documentURL, testname, true);
+    // Set the client visible area
+    sendTextFrame(socket1, "clientvisiblearea x=-4005 y=0 width=50490 height=72300");
+    sendTextFrame(socket1, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3840 tiletwipheight=3840");
+
+    //1. First make the first client to trigger the kit to filter out tiles based on identical wireIDs
+
+    // Type one character to trigger invalidation
+    sendChar(socket1, 'x', skNone, testname);
+
+    // First wsd forwards the invalidation
+    assertResponseString(socket1, "invalidatetiles:", testname);
+
+    // For the first input wsd will send all invalidated tiles
+    int arrivedTiles = 0;
+    bool gotTile = false;
+    do
+    {
+        std::vector<char> tile = getResponseMessage(socket1, "tile:", testname, 5000);
+        gotTile = !tile.empty();
+        if(gotTile)
+            ++arrivedTiles;
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_MESSAGE("We expect two tiles at least!", arrivedTiles > 1);
+
+    // Type an other character
+    sendChar(socket1, 'x', skNone, testname);
+    assertResponseString(socket1, "invalidatetiles:", testname);
+
+    // For the second input wsd will send one tile, since other tiles are indentical
+    arrivedTiles = 0;
+    gotTile = false;
+    do
+    {
+        std::vector<char> tile = getResponseMessage(socket1, "tile:", testname, 1000);
+        gotTile = !tile.empty();
+        if(gotTile)
+            ++arrivedTiles;
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_EQUAL(1, arrivedTiles);
+
+    //2. Now request the same tiles by the other client (e.g. scroll to the same view)
+
+    sendTextFrame(socket2, "tilecombine part=0 width=256 height=256 tileposx=0,3840,7680 tileposy=0,0,0 tilewidth=3840 tileheight=3840");
+
+    // We expect three tiles sent to the second client
+    arrivedTiles = 0;
+    gotTile = false;
+    do
+    {
+        std::vector<char> tile = getResponseMessage(socket2, "tile:", testname, 1000);
+        gotTile = !tile.empty();
+        if(gotTile)
+            ++arrivedTiles;
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_EQUAL(3, arrivedTiles);
+
+    // wsd should not send tiles messages for the first client
+    std::vector<char> tile = getResponseMessage(socket1, "tile:", testname, 1000);
+    CPPUNIT_ASSERT_MESSAGE("Not expected tile message arrived!", tile.empty());
+}
+
+void TileCacheTests::testLimitTileVersionsOnFly()
+{
+    // We have an upper limit (2) for the versions of the same tile wsd send out
+    // without getting the tileprocessed message for the first tile message.
+    const char* testname = "testLimitTileVersionsOnFly ";
+
+    std::string documentPath, documentURL;
+    getDocumentPathAndURL("empty.odt", documentPath, documentURL, testname);
+    std::shared_ptr<LOOLWebSocket> socket = loadDocAndGetSocket(_uri, documentURL, testname);
+
+    // Set the client visible area
+    sendTextFrame(socket, "clientvisiblearea x=-2662 y=0 width=16000 height=9875");
+    sendTextFrame(socket, "clientzoom tilepixelwidth=256 tilepixelheight=256 tiletwipwidth=3200 tiletwipheight=3200");
+
+    // Type one character to trigger sending tiles
+    sendChar(socket, 'x', skNone, testname);
+
+    // Handle all tiles send by wsd
+    bool getTileResp = false;
+    do
+    {
+        std::string tile = getResponseString(socket, "tile:", testname, 1000);
+        getTileResp = !tile.empty();
+    } while(getTileResp);
+
+    // Type an other character to trigger sending tiles
+    sendChar(socket, 'x', skNone, testname);
+
+    // Handle all tiles sent by wsd
+    getTileResp = false;
+    do
+    {
+        std::string tile = getResponseString(socket, "tile:", testname, 1000);
+        getTileResp = !tile.empty();
+    } while(getTileResp);
+
+    // For the third invalidation wsd does not send the new tile since
+    // two versions of the same tile were already sent.
+    sendChar(socket, 'x', skNone, testname);
+
+    std::vector<char> tile1 = getResponseMessage(socket, "tile:", testname, 1000);
+    CPPUNIT_ASSERT_MESSAGE("Not expected tile message arrived!", tile1.empty());
+
+    // When the next tileprocessed message arrive with correct tileID
+    // wsd sends the delayed tile
+    sendTextFrame(socket, "tileprocessed tile=0:0:0:3200:3200");
+
+    int arrivedTiles = 0;
+    bool gotTile = false;
+    do
+    {
+        std::vector<char> tile = getResponseMessage(socket, "tile:", testname, 1000);
+        gotTile = !tile.empty();
+        if(gotTile)
+            ++arrivedTiles;
+    } while(gotTile);
+
+    CPPUNIT_ASSERT_EQUAL(1, arrivedTiles);
+}
+
 CPPUNIT_TEST_SUITE_REGISTRATION(TileCacheTests);
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
