@@ -30,8 +30,6 @@
 #include <Poco/Thread.h>
 #include <Poco/URI.h>
 #include <Poco/Util/Application.h>
-#include <Poco/Util/HelpFormatter.h>
-#include <Poco/Util/Option.h>
 #include <Poco/Util/OptionSet.h>
 
 #include <Common.hpp>
@@ -68,8 +66,6 @@ using Poco::Runnable;
 using Poco::Thread;
 using Poco::URI;
 using Poco::Util::Application;
-using Poco::Util::HelpFormatter;
-using Poco::Util::Option;
 using Poco::Util::OptionSet;
 
 /// Thread class which performs the conversion.
@@ -91,17 +87,15 @@ public:
 
     void convertFile(const std::string& document)
     {
-        std::cerr << "convert file " << document << "\n";
-
         Poco::URI uri(_app._serverURI);
 
         Poco::Net::HTTPClientSession *session;
-        if (_app._serverURI.compare(0, 5, "https"))
+        if (_app._serverURI.compare(0, 5, "https") == 0)
             session = new Poco::Net::HTTPSClientSession(uri.getHost(), uri.getPort());
         else
             session = new Poco::Net::HTTPClientSession(uri.getHost(), uri.getPort());
 
-        Poco::Net::HTTPRequest request(Poco::Net::HTTPRequest::HTTP_POST, "/convert-to");
+        Poco::Net::HTTPRequest request(Poco::Net::HTTPRequest::HTTP_POST, "/lool/convert-to");
 
         try {
             Poco::Net::HTMLForm form;
@@ -123,22 +117,14 @@ public:
         Poco::Net::HTTPResponse response;
 
         try {
-            std::cerr << "try to get response\n";
-
             // receiveResponse() resulted in a Poco::Net::NoMessageException.
             std::istream& responseStream = session->receiveResponse(response);
 
-            std::cerr << "Get response\n";
-
             Poco::Path path(document);
             std::string outPath = _app._destinationDir + "/" + path.getBaseName() + "." + _app._destinationFormat;
             std::ofstream fileStream(outPath);
 
-            std::cerr << "write to " << outPath << "\n";
-
             Poco::StreamCopier::copyStream(responseStream, fileStream);
-
-            std::cerr << "Copied stream\n";
         }
         catch (const Poco::Exception &e)
         {
@@ -164,51 +150,40 @@ Tool::Tool() :
 
 void Tool::displayHelp()
 {
-    HelpFormatter helpFormatter(options());
-    helpFormatter.setCommand(commandName());
-    helpFormatter.setUsage("OPTIONS FILE(S)");
-    helpFormatter.setHeader("LibreOffice Online document converter tool.");
-    helpFormatter.format(std::cout);
+    std::cout << "LibreOffice Online document converter tool.\n"
+              << "Usage: " << commandName() << " [options] file...\n"
+              << "Options are:\n"
+              << "  --help                      Show this text\n"
+              << "  --extension=format          File format to convert to\n"
+              << "  --outdir=directory          Output directory for converted files\n"
+              << "  --parallelism=threads       Number of simultaneous threads to use\n"
+              << "  --server=uri                URI of LOOL server\n"
+              << "  --no-check-certificate      Disable checking of SSL certificate\n"
+              << "In addition, the options taken by the libreoffice command for its --convert-to\n"
+              << "functionality can be used (but are ignored if irrelevant to this command)." << std::endl;
 }
 
-void Tool::defineOptions(OptionSet& optionSet)
+void Tool::defineOptions(OptionSet&)
 {
-    Application::defineOptions(optionSet);
-
-    optionSet.addOption(Option("help", "", "Display help information on command line arguments.")
-                        .required(false).repeatable(false));
-    optionSet.addOption(Option("extension", "", "file format extension to convert to")
-                        .required(false).repeatable(false)
-                        .argument("format"));
-    optionSet.addOption(Option("outdir", "", "output directory for converted files")
-                        .required(false).repeatable(false).argument("outdir"));
-    optionSet.addOption(Option("parallelism", "", "number of simultaneous threads to use")
-                        .required(false).repeatable(false)
-                        .argument("threads"));
-    optionSet.addOption(Option("server", "", "URI of LOOL server")
-                        .required(false).repeatable(false)
-                        .argument("uri"));
-    optionSet.addOption(Option("no-check-certificate", "", "Disable checking of SSL certs")
-                        .required(false).repeatable(false));
+    stopOptionsProcessing();
 }
 
 void Tool::handleOption(const std::string& optionName,
                         const std::string& value)
 {
-    Application::handleOption(optionName, value);
-
     if (optionName == "help")
     {
         displayHelp();
         std::exit(Application::EXIT_OK);
     }
-    else if (optionName == "extension")
+    else if (optionName == "extension"
+             || optionName == "convert-to")
         _destinationFormat = value;
     else if (optionName == "outdir")
         _destinationDir = value;
     else if (optionName == "parallelism")
         _numWorkers = std::max(std::stoi(value), 1);
-    else if (optionName == "uri")
+    else if (optionName == "server")
         _serverURI = value;
     else if (optionName == "no-check-certificate")
     {
@@ -219,8 +194,49 @@ void Tool::handleOption(const std::string& optionName,
     }
 }
 
-int Tool::main(const std::vector<std::string>& args)
+int Tool::main(const std::vector<std::string>& origArgs)
 {
+    std::vector<std::string> args = origArgs;
+
+    for (unsigned i = 0; i < origArgs.size(); ++i)
+    {
+        if (origArgs[i].length() > 0 && origArgs[i][0] != '-')
+            break;
+
+        // It's an option. Erase it from the file name vector.
+        args.erase(args.begin());
+
+        std::string optionName, value;
+
+        // Accept either one or two dashes, like LibreOffice.
+        if (origArgs[i].length() > 1 && origArgs[i][1] != '-')
+            optionName = origArgs[i].substr(1);
+        else if (origArgs[i].length() > 1 && origArgs[i][1] == '-')
+            optionName = origArgs[i].substr(2);
+        else
+            break;
+
+        std::string::size_type equals = optionName.find('=');
+
+        // Handle LibreOffice-compatible options that don't have their value separated with an equals,
+        // but as the next argument.
+        if (equals == std::string::npos
+            && (optionName == "convert-to"
+                || optionName == "outdir")
+            && i < origArgs.size()-1)
+        {
+            value = origArgs[i+1];
+            args.erase(args.begin());
+            ++i;
+        }
+        else if (equals != std::string::npos)
+        {
+            value = optionName.substr(equals+1);
+            optionName = optionName.substr(0, equals);
+        }
+        handleOption(optionName, value);
+    }
+
     if (args.empty())
     {
         std::cerr << "Nothing to do." << std::endl;
