@@ -13,6 +13,7 @@
 
 #include <algorithm>
 #include <cassert>
+#include <errno.h>
 #include <fstream>
 #include <iconv.h>
 #include <string>
@@ -22,37 +23,44 @@
 #include <Poco/Exception.h>
 #include <Poco/JSON/Object.h>
 #include <Poco/JSON/Parser.h>
+
+#ifndef MOBILEAPP
+
+#include <Poco/Net/AcceptCertificateHandler.h>
+#include <Poco/Net/Context.h>
 #include <Poco/Net/DNS.h>
 #include <Poco/Net/HTTPClientSession.h>
 #include <Poco/Net/HTTPRequest.h>
 #include <Poco/Net/HTTPResponse.h>
 #include <Poco/Net/HTTPSClientSession.h>
+#include <Poco/Net/KeyConsoleHandler.h>
 #include <Poco/Net/NameValueCollection.h>
 #include <Poco/Net/NetworkInterface.h>
 #include <Poco/Net/SSLManager.h>
+
+#endif
+
 #include <Poco/StreamCopier.h>
 #include <Poco/Timestamp.h>
-
-// For residual Poco SSL usage.
-#include <Poco/Net/AcceptCertificateHandler.h>
-#include <Poco/Net/Context.h>
-#include <Poco/Net/KeyConsoleHandler.h>
-#include <Poco/Net/SSLManager.h>
+#include <Poco/URI.h>
 
 #include "Auth.hpp"
 #include <Common.hpp>
 #include "Exceptions.hpp"
-#include "LOOLWSD.hpp"
 #include <Log.hpp>
 #include <Unit.hpp>
 #include <Util.hpp>
 #include <common/FileUtil.hpp>
 #include <common/JsonUtil.hpp>
 
+using std::size_t;
+
 bool StorageBase::FilesystemEnabled;
 bool StorageBase::WopiEnabled;
 Util::RegexListMatcher StorageBase::WopiHosts;
 
+#ifndef MOBILEAPP
+
 std::string StorageBase::getLocalRootPath() const
 {
     std::string localPath = _jailPath;
@@ -74,8 +82,11 @@ size_t StorageBase::getFileSize(const std::string& filename)
     return std::ifstream(filename, std::ifstream::ate | std::ifstream::binary).tellg();
 }
 
+#endif
+
 void StorageBase::initialize()
 {
+#ifndef MOBILEAPP
     const auto& app = Poco::Util::Application::instance();
     FilesystemEnabled = app.config().getBool("storage.filesystem[@allow]", false);
 
@@ -125,8 +136,18 @@ void StorageBase::initialize()
     Poco::Net::Context::Ptr sslClientContext = new Poco::Net::Context(Poco::Net::Context::CLIENT_USE, sslClientParams);
     Poco::Net::SSLManager::instance().initializeClient(consoleClientHandler, invalidClientCertHandler, sslClientContext);
 #endif
+#else
+    FilesystemEnabled = true;
+#endif
 }
 
+bool StorageBase::allowedWopiHost(const std::string& host)
+{
+    return WopiEnabled && WopiHosts.match(host);
+}
+
+#ifndef MOBILEAPP
+
 bool isLocalhost(const std::string& targetHost)
 {
     std::string targetAddress;
@@ -165,6 +186,8 @@ bool isLocalhost(const std::string& targetHost)
     return false;
 }
 
+#endif
+
 std::unique_ptr<StorageBase> StorageBase::create(const Poco::URI& uri, const std::string& jailRoot, const std::string& jailPath)
 {
     // FIXME: By the time this gets called we have already sent to the client three
@@ -184,7 +207,7 @@ std::unique_ptr<StorageBase> StorageBase::create(const Poco::URI& uri, const std
     }
     else if (uri.isRelative() || uri.getScheme() == "file")
     {
-        LOG_INF("Public URI [" << uri.toString() << "] is a file.");
+        LOG_INF("Public URI [" << LOOLWSD::anonymizeUrl(uri.toString()) << "] is a file.");
 
 #if ENABLE_DEBUG
         if (std::getenv("FAKE_UNAUTHORIZED"))
@@ -208,16 +231,17 @@ std::unique_ptr<StorageBase> StorageBase::create(const Poco::URI& uri, const std
 
             if (pathSegments.size() == 4 && pathSegments[0] == "tmp" && pathSegments[1] == "convert-to")
             {
-                LOG_INF("Public URI [" << normalizedUri.toString() << "] is actually a convert-to tempfile.");
+                LOG_INF("Public URI [" << LOOLWSD::anonymizeUrl(normalizedUri.toString()) << "] is actually a convert-to tempfile.");
                 return std::unique_ptr<StorageBase>(new LocalStorage(normalizedUri, jailRoot, jailPath));
             }
         }
 
         LOG_ERR("Local Storage is disabled by default. Enable in the config file or on the command-line to enable.");
     }
+#ifndef MOBILEAPP
     else if (WopiEnabled)
     {
-        LOG_INF("Public URI [" << uri.toString() << "] considered WOPI.");
+        LOG_INF("Public URI [" << LOOLWSD::anonymizeUrl(uri.toString()) << "] considered WOPI.");
         const auto& targetHost = uri.getHost();
         if (WopiHosts.match(targetHost) || isLocalhost(targetHost))
         {
@@ -226,7 +250,7 @@ std::unique_ptr<StorageBase> StorageBase::create(const Poco::URI& uri, const std
         LOG_ERR("No acceptable WOPI hosts found matching the target host [" << targetHost << "] in config.");
         throw UnauthorizedRequestException("No acceptable WOPI hosts found matching the target host [" + targetHost + "] in config.");
     }
-
+#endif
     throw BadRequestException("No Storage configured or invalid URI.");
 }
 
@@ -235,7 +259,7 @@ std::atomic<unsigned> LocalStorage::LastLocalStorageId;
 std::unique_ptr<LocalStorage::LocalFileInfo> LocalStorage::getLocalFileInfo()
 {
     const Poco::Path path = Poco::Path(_uri.getPath());
-    LOG_DBG("Getting info for local uri [" << _uri.toString() << "], path [" << path.toString() << "].");
+    LOG_DBG("Getting info for local uri [" << LOOLWSD::anonymizeUrl(_uri.toString()) << "], path [" << LOOLWSD::anonymizeUrl(path.toString()) << "].");
 
     const auto& filename = path.getFileName();
     const Poco::File file = Poco::File(path);
@@ -245,31 +269,33 @@ std::unique_ptr<LocalStorage::LocalFileInfo> LocalStorage::getLocalFileInfo()
     _fileInfo = FileInfo({filename, "localhost", lastModified, size});
 
     // Set automatic userid and username
-    return std::unique_ptr<LocalStorage::LocalFileInfo>(new LocalFileInfo({"localhost" + std::to_string(LastLocalStorageId), "Local Host #" + std::to_string(LastLocalStorageId++)}));
+    return std::unique_ptr<LocalStorage::LocalFileInfo>(new LocalFileInfo({"localhost" + std::to_string(LastLocalStorageId), "LocalHost#" + std::to_string(LastLocalStorageId++)}));
 }
 
 std::string LocalStorage::loadStorageFileToLocal(const Authorization& /*auth*/)
 {
+#ifndef MOBILEAPP
     // /chroot/jailId/user/doc/childId/file.ext
     const std::string filename = Poco::Path(_uri.getPath()).getFileName();
     _jailedFilePath = Poco::Path(getLocalRootPath(), filename).toString();
-    LOG_INF("Public URI [" << _uri.getPath() <<
-            "] jailed to [" << _jailedFilePath << "].");
+    _jailedFilePathAnonym = LOOLWSD::anonymizeUrl(_jailedFilePath);
+    LOG_INF("Public URI [" << LOOLWSD::anonymizeUrl(_uri.getPath()) <<
+            "] jailed to [" << _jailedFilePathAnonym << "].");
 
     // Despite the talk about URIs it seems that _uri is actually just a pathname here
     const std::string publicFilePath = _uri.getPath();
 
     if (!FileUtil::checkDiskSpace(_jailedFilePath))
     {
-        throw StorageSpaceLowException("Low disk space for " + _jailedFilePath);
+        throw StorageSpaceLowException("Low disk space for " + _jailedFilePathAnonym);
     }
 
-    LOG_INF("Linking " << publicFilePath << " to " << _jailedFilePath);
+    LOG_INF("Linking " << LOOLWSD::anonymizeUrl(publicFilePath) << " to " << _jailedFilePathAnonym);
     if (!Poco::File(_jailedFilePath).exists() && link(publicFilePath.c_str(), _jailedFilePath.c_str()) == -1)
     {
         // Failed
-        LOG_WRN("link(\"" << publicFilePath << "\", \"" << _jailedFilePath << "\") failed. Will copy. "
-                "Linking error: " << errno << " " << strerror(errno));
+        LOG_WRN("link(\"" << LOOLWSD::anonymizeUrl(publicFilePath) << "\", \"" << _jailedFilePathAnonym << "\") failed. Will copy. "
+                "Linking error: " << Util::symbolicErrno(errno) << " " << strerror(errno));
     }
 
     try
@@ -277,14 +303,14 @@ std::string LocalStorage::loadStorageFileToLocal(const Authorization& /*auth*/)
         // Fallback to copying.
         if (!Poco::File(_jailedFilePath).exists())
         {
-            LOG_INF("Copying " << publicFilePath << " to " << _jailedFilePath);
+            LOG_INF("Copying " << LOOLWSD::anonymizeUrl(publicFilePath) << " to " << _jailedFilePathAnonym);
             Poco::File(publicFilePath).copyTo(_jailedFilePath);
             _isCopy = true;
         }
     }
     catch (const Poco::Exception& exc)
     {
-        LOG_ERR("copyTo(\"" << publicFilePath << "\", \"" << _jailedFilePath << "\") failed: " << exc.displayText());
+        LOG_ERR("copyTo(\"" << LOOLWSD::anonymizeUrl(publicFilePath) << "\", \"" << _jailedFilePathAnonym << "\") failed: " << exc.displayText());
         throw;
     }
 
@@ -298,17 +324,27 @@ std::string LocalStorage::loadStorageFileToLocal(const Authorization& /*auth*/)
 #else
     return _jailedFilePath;
 #endif
+
+#else // MOBILEAPP
+
+    // In the mobile app we use no jail
+    _jailedFilePath = _uri.getPath();
+    _isLoaded = true;
+
+    return _jailedFilePath;
+#endif
+
 }
 
 StorageBase::SaveResult LocalStorage::saveLocalFileToStorage(const Authorization& /*auth*/, const std::string& /*saveAsPath*/, const std::string& /*saveAsFilename*/)
 {
     try
     {
-        LOG_TRC("Saving local file to local file storage (isCopy: " << _isCopy << ") for " << _jailedFilePath);
+        LOG_TRC("Saving local file to local file storage (isCopy: " << _isCopy << ") for " << _jailedFilePathAnonym);
         // Copy the file back.
         if (_isCopy && Poco::File(_jailedFilePath).exists())
         {
-            LOG_INF("Copying " << _jailedFilePath << " to " << _uri.getPath());
+            LOG_INF("Copying " << _jailedFilePathAnonym << " to " << LOOLWSD::anonymizeUrl(_uri.getPath()));
             Poco::File(_jailedFilePath).copyTo(_uri.getPath());
         }
 
@@ -319,7 +355,7 @@ StorageBase::SaveResult LocalStorage::saveLocalFileToStorage(const Authorization
     }
     catch (const Poco::Exception& exc)
     {
-        LOG_ERR("copyTo(\"" << _jailedFilePath << "\", \"" << _uri.getPath() <<
+        LOG_ERR("copyTo(\"" << _jailedFilePathAnonym << "\", \"" << LOOLWSD::anonymizeUrl(_uri.getPath()) <<
                 "\") failed: " << exc.displayText());
         return StorageBase::SaveResult::FAILED;
     }
@@ -327,6 +363,8 @@ StorageBase::SaveResult LocalStorage::saveLocalFileToStorage(const Authorization
     return StorageBase::SaveResult(StorageBase::SaveResult::OK);
 }
 
+#ifndef MOBILEAPP
+
 namespace
 {
 
@@ -364,10 +402,13 @@ Poco::Timestamp iso8601ToTimestamp(const std::string& iso8601Time, const std::st
     Poco::Timestamp timestamp = Poco::Timestamp::fromEpochTime(0);
     try
     {
-        int timeZoneDifferential;
-        Poco::DateTime dateTime;
-        Poco::DateTimeParser::parse(Poco::DateTimeFormat::ISO8601_FRAC_FORMAT, iso8601Time, dateTime, timeZoneDifferential);
-        timestamp = dateTime.timestamp();
+        if (!iso8601Time.empty())
+        {
+            int timeZoneDifferential;
+            Poco::DateTime dateTime;
+            Poco::DateTimeParser::parse(Poco::DateTimeFormat::ISO8601_FRAC_FORMAT, iso8601Time, dateTime, timeZoneDifferential);
+            timestamp = dateTime.timestamp();
+        }
     }
     catch (const Poco::SyntaxException& exc)
     {
@@ -385,20 +426,22 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
     // update the access_token to the one matching to the session
     Poco::URI uriObject(_uri);
     auth.authorizeURI(uriObject);
+    const std::string uriAnonym = LOOLWSD::anonymizeUrl(uriObject.toString());
 
-    LOG_DBG("Getting info for wopi uri [" << uriObject.toString() << "].");
+    LOG_DBG("Getting info for wopi uri [" << uriAnonym << "].");
 
-    std::string resMsg;
-    const auto startTime = std::chrono::steady_clock::now();
+    std::string wopiResponse;
     std::chrono::duration<double> callDuration(0);
     try
     {
-        std::unique_ptr<Poco::Net::HTTPClientSession> psession(getHTTPClientSession(uriObject));
-
         Poco::Net::HTTPRequest request(Poco::Net::HTTPRequest::HTTP_GET, uriObject.getPathAndQuery(), Poco::Net::HTTPMessage::HTTP_1_1);
         request.set("User-Agent", WOPI_AGENT_STRING);
         auth.authorizeRequest(request);
         addStorageDebugCookie(request);
+
+        const auto startTime = std::chrono::steady_clock::now();
+
+        std::unique_ptr<Poco::Net::HTTPClientSession> psession(getHTTPClientSession(uriObject));
         psession->sendRequest(request);
 
         Poco::Net::HTTPResponse response;
@@ -408,13 +451,13 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
         Log::StreamLogger logger = Log::trace();
         if (logger.enabled())
         {
-            logger << "WOPI::CheckFileInfo header for URI [" << uriObject.toString() << "]:\n";
+            logger << "WOPI::CheckFileInfo header for URI [" << uriAnonym << "]:\n";
             for (const auto& pair : response)
             {
                 logger << '\t' << pair.first << ": " << pair.second << " / ";
             }
 
-            LOG_END(logger);
+            LOG_END(logger, true);
         }
 
         if (response.getStatus() != Poco::Net::HTTPResponse::HTTP_OK)
@@ -423,12 +466,12 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
             throw StorageConnectionException("WOPI::CheckFileInfo failed");
         }
 
-        Poco::StreamCopier::copyToString(rs, resMsg);
+        Poco::StreamCopier::copyToString(rs, wopiResponse);
     }
-    catch(const Poco::Exception& pexc)
+    catch (const Poco::Exception& pexc)
     {
-        LOG_ERR("Cannot get file info from WOPI storage uri [" << uriObject.toString() << "]. Error: " << pexc.displayText() <<
-                (pexc.nested() ? " (" + pexc.nested()->displayText() + ")" : ""));
+        LOG_ERR("Cannot get file info from WOPI storage uri [" << uriAnonym << "]. Error: " <<
+                pexc.displayText() << (pexc.nested() ? " (" + pexc.nested()->displayText() + ")" : ""));
         throw;
     }
 
@@ -438,6 +481,7 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
     std::string ownerId;
     std::string userId;
     std::string userName;
+    std::string obfuscatedUserId;
     std::string userExtraInfo;
     std::string watermarkText;
     bool canWrite = false;
@@ -446,23 +490,78 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
     bool hidePrintOption = false;
     bool hideSaveOption = false;
     bool hideExportOption = false;
-    bool hideChangeTrackingControls = false;
     bool disablePrint = false;
     bool disableExport = false;
     bool disableCopy = false;
     bool disableInactiveMessages = false;
     std::string lastModifiedTime;
     bool userCanNotWriteRelative = true;
+    bool enableInsertRemoteImage = false;
+    bool enableShare = false;
+    WOPIFileInfo::TriState disableChangeTrackingRecord = WOPIFileInfo::TriState::Unset;
+    WOPIFileInfo::TriState disableChangeTrackingShow = WOPIFileInfo::TriState::Unset;
+    WOPIFileInfo::TriState hideChangeTrackingControls = WOPIFileInfo::TriState::Unset;
 
-    LOG_DBG("WOPI::CheckFileInfo returned: " << resMsg << ". Call duration: " << callDuration.count() << "s");
     Poco::JSON::Object::Ptr object;
-    if (JsonUtil::parseJSON(resMsg, object))
+    if (JsonUtil::parseJSON(wopiResponse, object))
     {
+        if (LOOLWSD::AnonymizeFilenames || LOOLWSD::AnonymizeUsernames)
+            LOG_DBG("WOPI::CheckFileInfo (" << callDuration.count() * 1000. << " ms): anonymizing...");
+        else
+            LOG_DBG("WOPI::CheckFileInfo (" << callDuration.count() * 1000. << " ms): " << wopiResponse);
+
         JsonUtil::findJSONValue(object, "BaseFileName", filename);
-        JsonUtil::findJSONValue(object, "Size", size);
         JsonUtil::findJSONValue(object, "OwnerId", ownerId);
         JsonUtil::findJSONValue(object, "UserId", userId);
         JsonUtil::findJSONValue(object, "UserFriendlyName", userName);
+
+        // Anonymize key values.
+        if (LOOLWSD::AnonymizeFilenames || LOOLWSD::AnonymizeUsernames)
+        {
+            Util::mapAnonymized(Util::getFilenameFromURL(filename), Util::getFilenameFromURL(_uri.toString()));
+
+            JsonUtil::findJSONValue(object, "ObfuscatedUserId", obfuscatedUserId, false);
+            if (!obfuscatedUserId.empty())
+            {
+                Util::mapAnonymized(ownerId, obfuscatedUserId);
+                Util::mapAnonymized(userId, obfuscatedUserId);
+                Util::mapAnonymized(userName, obfuscatedUserId);
+            }
+
+            // Set anonymized version of the above fields before logging.
+            // Note: anonymization caches the result, so we don't need to store here.
+            if (LOOLWSD::AnonymizeFilenames)
+                object->set("BaseFileName", LOOLWSD::anonymizeUrl(filename));
+
+            // If obfuscatedUserId is provided, then don't log the originals and use it.
+            if (LOOLWSD::AnonymizeUsernames && obfuscatedUserId.empty())
+            {
+                object->set("OwnerId", LOOLWSD::anonymizeUsername(ownerId));
+                object->set("UserId", LOOLWSD::anonymizeUsername(userId));
+                object->set("UserFriendlyName", LOOLWSD::anonymizeUsername(userName));
+            }
+
+            std::ostringstream oss;
+            object->stringify(oss);
+            wopiResponse = oss.str();
+
+            // Remove them for performance reasons; they aren't needed anymore.
+            object->remove("ObfuscatedUserId");
+
+            if (LOOLWSD::AnonymizeFilenames)
+                object->remove("BaseFileName");
+
+            if (LOOLWSD::AnonymizeUsernames)
+            {
+                object->remove("OwnerId");
+                object->remove("UserId");
+                object->remove("UserFriendlyName");
+            }
+
+            LOG_DBG("WOPI::CheckFileInfo (" << callDuration.count() * 1000. << " ms): " << wopiResponse);
+        }
+
+        JsonUtil::findJSONValue(object, "Size", size);
         JsonUtil::findJSONValue(object, "UserExtraInfo", userExtraInfo);
         JsonUtil::findJSONValue(object, "WatermarkText", watermarkText);
         JsonUtil::findJSONValue(object, "UserCanWrite", canWrite);
@@ -470,7 +569,6 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
         JsonUtil::findJSONValue(object, "HidePrintOption", hidePrintOption);
         JsonUtil::findJSONValue(object, "HideSaveOption", hideSaveOption);
         JsonUtil::findJSONValue(object, "HideExportOption", hideExportOption);
-        JsonUtil::findJSONValue(object, "hideChangeTrackingControls", hideChangeTrackingControls);
         JsonUtil::findJSONValue(object, "EnableOwnerTermination", enableOwnerTermination);
         JsonUtil::findJSONValue(object, "DisablePrint", disablePrint);
         JsonUtil::findJSONValue(object, "DisableExport", disableExport);
@@ -478,17 +576,38 @@ std::unique_ptr<WopiStorage::WOPIFileInfo> WopiStorage::getWOPIFileInfo(const Au
         JsonUtil::findJSONValue(object, "DisableInactiveMessages", disableInactiveMessages);
         JsonUtil::findJSONValue(object, "LastModifiedTime", lastModifiedTime);
         JsonUtil::findJSONValue(object, "UserCanNotWriteRelative", userCanNotWriteRelative);
+        JsonUtil::findJSONValue(object, "EnableInsertRemoteImage", enableInsertRemoteImage);
+        JsonUtil::findJSONValue(object, "EnableShare", enableShare);
+        bool booleanFlag = false;
+        if (JsonUtil::findJSONValue(object, "DisableChangeTrackingRecord", booleanFlag))
+            disableChangeTrackingRecord = (booleanFlag ? WOPIFileInfo::TriState::True : WOPIFileInfo::TriState::False);
+        if (JsonUtil::findJSONValue(object, "DisableChangeTrackingShow", booleanFlag))
+            disableChangeTrackingShow = (booleanFlag ? WOPIFileInfo::TriState::True : WOPIFileInfo::TriState::False);
+        if (JsonUtil::findJSONValue(object, "HideChangeTrackingControls", booleanFlag))
+            hideChangeTrackingControls = (booleanFlag ? WOPIFileInfo::TriState::True : WOPIFileInfo::TriState::False);
     }
     else
     {
-        LOG_ERR("WOPI::CheckFileInfo failed and no JSON payload returned. Access denied.");
-        throw UnauthorizedRequestException("Access denied. WOPI::CheckFileInfo failed on: " + uriObject.toString());
+        if (LOOLWSD::AnonymizeFilenames || LOOLWSD::AnonymizeUsernames)
+            wopiResponse = "obfuscated";
+
+        LOG_ERR("WOPI::CheckFileInfo (" << callDuration.count() * 1000. <<
+                " ms) failed or no valid JSON payload returned. Access denied. "
+                "Original response: [" << wopiResponse << "].");
+
+        throw UnauthorizedRequestException("Access denied. WOPI::CheckFileInfo failed on: " + uriAnonym);
     }
 
     const Poco::Timestamp modifiedTime = iso8601ToTimestamp(lastModifiedTime, "LastModifiedTime");
     _fileInfo = FileInfo({filename, ownerId, modifiedTime, size});
 
-    return std::unique_ptr<WopiStorage::WOPIFileInfo>(new WOPIFileInfo({userId, userName, userExtraInfo, watermarkText, canWrite, postMessageOrigin, hidePrintOption, hideSaveOption, hideExportOption, hideChangeTrackingControls, enableOwnerTermination, disablePrint, disableExport, disableCopy, disableInactiveMessages, userCanNotWriteRelative, callDuration}));
+    return std::unique_ptr<WopiStorage::WOPIFileInfo>(new WOPIFileInfo(
+        {userId, obfuscatedUserId, userName, userExtraInfo, watermarkText, canWrite,
+         postMessageOrigin, hidePrintOption, hideSaveOption, hideExportOption,
+         enableOwnerTermination, disablePrint, disableExport, disableCopy,
+         disableInactiveMessages, userCanNotWriteRelative, enableInsertRemoteImage, enableShare,
+         disableChangeTrackingShow, disableChangeTrackingRecord,
+         hideChangeTrackingControls, callDuration}));
 }
 
 /// uri format: http://server/<...>/wopi*/files/<id>/content
@@ -500,7 +619,11 @@ std::string WopiStorage::loadStorageFileToLocal(const Authorization& auth)
     uriObject.setPath(uriObject.getPath() + "/contents");
     auth.authorizeURI(uriObject);
 
-    LOG_DBG("Wopi requesting: " << uriObject.toString());
+    Poco::URI uriObjectAnonym(_uri);
+    uriObjectAnonym.setPath(LOOLWSD::anonymizeUrl(uriObjectAnonym.getPath()) + "/contents");
+    const std::string uriAnonym = uriObjectAnonym.toString();
+
+    LOG_DBG("Wopi requesting: " << uriAnonym);
 
     const auto startTime = std::chrono::steady_clock::now();
     try
@@ -521,13 +644,13 @@ std::string WopiStorage::loadStorageFileToLocal(const Authorization& auth)
         Log::StreamLogger logger = Log::trace();
         if (logger.enabled())
         {
-            logger << "WOPI::GetFile header for URI [" << uriObject.toString() << "]:\n";
+            logger << "WOPI::GetFile header for URI [" << uriAnonym << "]:\n";
             for (const auto& pair : response)
             {
                 logger << '\t' << pair.first << ": " << pair.second << " / ";
             }
 
-            LOG_END(logger);
+            LOG_END(logger, true);
         }
 
         if (response.getStatus() != Poco::Net::HTTPResponse::HTTP_OK)
@@ -538,23 +661,24 @@ std::string WopiStorage::loadStorageFileToLocal(const Authorization& auth)
         else // Successful
         {
             _jailedFilePath = Poco::Path(getLocalRootPath(), _fileInfo._filename).toString();
+            _jailedFilePathAnonym = LOOLWSD::anonymizeUrl(_jailedFilePath);
             std::ofstream ofs(_jailedFilePath);
             std::copy(std::istreambuf_iterator<char>(rs),
                       std::istreambuf_iterator<char>(),
                       std::ostreambuf_iterator<char>(ofs));
             ofs.close();
-            LOG_INF("WOPI::GetFile downloaded " << getFileSize(_jailedFilePath) << " bytes from [" << uriObject.toString() <<
-                    "] -> " << _jailedFilePath << " in " << diff.count() << "s");
+            LOG_INF("WOPI::GetFile downloaded " << getFileSize(_jailedFilePath) << " bytes from [" <<
+                    uriAnonym << "] -> " << _jailedFilePathAnonym << " in " << diff.count() << "s");
 
             _isLoaded = true;
             // Now return the jailed path.
             return Poco::Path(_jailPath, _fileInfo._filename).toString();
         }
     }
-    catch(const Poco::Exception& pexc)
+    catch (const Poco::Exception& pexc)
     {
-        LOG_ERR("Cannot load document from WOPI storage uri [" + uriObject.toString() + "]. Error: " << pexc.displayText() <<
-                (pexc.nested() ? " (" + pexc.nested()->displayText() + ")" : ""));
+        LOG_ERR("Cannot load document from WOPI storage uri [" + uriAnonym + "]. Error: " <<
+                pexc.displayText() << (pexc.nested() ? " (" + pexc.nested()->displayText() + ")" : ""));
         throw;
     }
 
@@ -566,17 +690,18 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
     // TODO: Check if this URI has write permission (canWrite = true)
 
     const bool isSaveAs = !saveAsPath.empty() && !saveAsFilename.empty();
-    const std::string filePath(isSaveAs? saveAsPath: _jailedFilePath);
+    const std::string filePath(isSaveAs ? saveAsPath : _jailedFilePath);
+    const std::string filePathAnonym = LOOLWSD::anonymizeUrl(filePath);
 
     const size_t size = getFileSize(filePath);
 
     Poco::URI uriObject(_uri);
     uriObject.setPath(isSaveAs? uriObject.getPath(): uriObject.getPath() + "/contents");
     auth.authorizeURI(uriObject);
+    const std::string uriAnonym = LOOLWSD::anonymizeUrl(uriObject.toString());
 
-    LOG_INF("Uploading URI via WOPI [" << uriObject.toString() << "] from [" << filePath + "].");
+    LOG_INF("Uploading URI via WOPI [" << uriAnonym << "] from [" << filePathAnonym + "].");
 
-    std::ostringstream oss;
     StorageBase::SaveResult saveResult(StorageBase::SaveResult::FAILED);
     try
     {
@@ -592,6 +717,7 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
             request.set("X-WOPI-Override", "PUT");
             request.set("X-LOOL-WOPI-IsModifiedByUser", _isUserModified? "true": "false");
             request.set("X-LOOL-WOPI-IsAutosave", _isAutosave? "true": "false");
+            request.set("X-LOOL-WOPI-IsExitSave", isExitSave()? "true": "false");
 
             if (!_forceSave)
             {
@@ -648,13 +774,48 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
 
         Poco::Net::HTTPResponse response;
         std::istream& rs = psession->receiveResponse(response);
+
+        std::ostringstream oss;
         Poco::StreamCopier::copyStream(rs, oss);
+        std::string responseString = oss.str();
 
         const std::string wopiLog(isSaveAs ? "WOPI::PutRelativeFile" : "WOPI::PutFile");
-        LOG_INF(wopiLog << " response: " << oss.str());
-        LOG_INF(wopiLog << " uploaded " << size << " bytes from [" << filePath <<
-                "] -> [" << uriObject.toString() << "]: " <<
-                response.getStatus() << " " << response.getReason());
+
+        if (Log::infoEnabled())
+        {
+            if (LOOLWSD::AnonymizeFilenames)
+            {
+                Poco::JSON::Object::Ptr object;
+                if (JsonUtil::parseJSON(responseString, object))
+                {
+                    // Anonymize the filename
+                    std::string url;
+                    std::string filename;
+                    if (JsonUtil::findJSONValue(object, "Url", url) &&
+                        JsonUtil::findJSONValue(object, "Name", filename))
+                    {
+                        // Get the FileId form the URL, which we use as the anonymized filename.
+                        std::string decodedUrl;
+                        Poco::URI::decode(url, decodedUrl);
+                        const std::string obfuscatedFileId = Util::getFilenameFromURL(decodedUrl);
+                        Util::mapAnonymized(obfuscatedFileId, obfuscatedFileId); // Identity, to avoid re-anonymizing.
+
+                        const std::string filenameOnly = Util::getFilenameFromURL(filename);
+                        Util::mapAnonymized(filenameOnly, obfuscatedFileId);
+                        object->set("Name", LOOLWSD::anonymizeUrl(filename));
+                    }
+
+                    // Stringify to log.
+                    std::ostringstream ossResponse;
+                    object->stringify(ossResponse);
+                    responseString = ossResponse.str();
+                }
+            }
+
+            LOG_INF(wopiLog << " response: " << responseString);
+            LOG_INF(wopiLog << " uploaded " << size << " bytes from [" << filePathAnonym <<
+                    "] -> [" << uriAnonym << "]: " << response.getStatus() << " " << response.getReason());
+        }
 
         if (response.getStatus() == Poco::Net::HTTPResponse::HTTP_OK)
         {
@@ -669,10 +830,10 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
                 if (isSaveAs)
                 {
                     const std::string name = JsonUtil::getJSONValue<std::string>(object, "Name");
-                    LOG_TRC(wopiLog << " returns Name [" << name << "].");
+                    LOG_TRC(wopiLog << " returns Name [" << LOOLWSD::anonymizeUrl(name) << "].");
 
                     const std::string url = JsonUtil::getJSONValue<std::string>(object, "Url");
-                    LOG_TRC(wopiLog << " returns Url [" << url << "].");
+                    LOG_TRC(wopiLog << " returns Url [" << LOOLWSD::anonymizeUrl(url) << "].");
 
                     saveResult.setSaveAsResult(name, url);
                 }
@@ -683,7 +844,7 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
             }
             else
             {
-                LOG_WRN("Invalid or missing JSON in " << wopiLog << " HTTP_OK response");
+                LOG_WRN("Invalid or missing JSON in " << wopiLog << " HTTP_OK response.");
             }
         }
         else if (response.getStatus() == Poco::Net::HTTPResponse::HTTP_REQUESTENTITYTOOLARGE)
@@ -708,14 +869,14 @@ StorageBase::SaveResult WopiStorage::saveLocalFileToStorage(const Authorization&
             }
             else
             {
-                LOG_WRN("Invalid or missing JSON in " << wopiLog << " HTTP_CONFLICT response");
+                LOG_WRN("Invalid or missing JSON in " << wopiLog << " HTTP_CONFLICT response.");
             }
         }
     }
-    catch(const Poco::Exception& pexc)
+    catch (const Poco::Exception& pexc)
     {
-        LOG_ERR("Cannot save file to WOPI storage uri [" + uriObject.toString() + "]. Error: " << pexc.displayText() <<
-                (pexc.nested() ? " (" + pexc.nested()->displayText() + ")" : ""));
+        LOG_ERR("Cannot save file to WOPI storage uri [" << uriAnonym << "]. Error: " <<
+                pexc.displayText() << (pexc.nested() ? " (" + pexc.nested()->displayText() + ")" : ""));
         saveResult.setResult(StorageBase::SaveResult::FAILED);
     }
 
@@ -735,4 +896,6 @@ StorageBase::SaveResult WebDAVStorage::saveLocalFileToStorage(const Authorizatio
     return StorageBase::SaveResult(StorageBase::SaveResult::OK);
 }
 
+#endif
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
